# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from django.test.client import Client
from django.utils.translation import ugettext as _

import okconfig
import adagios.settings

okconfig.cfg_file = adagios.settings.nagios_config


class TestOkconfig(TestCase):

    def testOkconfigVerifies(self):
        result = okconfig.verify()
        for k, v in result.items():
            self.assertTrue(v, msg=_("Failed on test: %s") % k)

    def testIndexPage(self):
        c = Client()
        response = c.get('/okconfig/verify_okconfig')
        self.assertEqual(response.status_code, 200)

    def testPageLoad(self):
        """ Smoketest for the okconfig views """
        self.loadPage('/okconfig/addhost')
        self.loadPage('/okconfig/scan_network')
        self.loadPage('/okconfig/addgroup')
        self.loadPage('/okconfig/addtemplate')
        self.loadPage('/okconfig/addhost')
        self.loadPage('/okconfig/addservice')
        self.loadPage('/okconfig/install_agent')
        self.loadPage('/okconfig/edit')
        self.loadPage('/okconfig/edit/localhost')
        self.loadPage('/okconfig/verify_okconfig')
    def loadPage(self, url):
        """ Load one specific page, and assert if return code is not 200 """
        try:
            c = Client()
            response = c.get(url)
            self.assertEqual(response.status_code, 200, _("Expected status code 200 for page %s") % url)
        except Exception, e:
            self.assertEqual(True, _("Unhandled exception while loading %(url)s: %(e)s") % {'url': url, 'e': e})



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import math
from datetime import datetime, timedelta
from django import template
from django.utils.timesince import timesince
from django.utils.translation import ugettext as _

register = template.Library()

@register.filter("timestamp")
def timestamp(value):
    try:
        return datetime.fromtimestamp(value)
    except AttributeError:
        return ''

@register.filter("duration")
def duration(value):
    """ Used as a filter, returns a human-readable duration.
    'value' must be in seconds.
    """
    zero = datetime.min
    return timesince(zero, zero + timedelta(0, value))

@register.filter("hash")
def hash(h, key):
    return h[key]





from django.test import LiveServerTestCase
from django.test import TestCase
import adagios.settings
import adagios.utils
import os

try:
    from selenium import webdriver
    from selenium.common.exceptions import WebDriverException
except ImportError:
    webdriver = None

def get_remote_webdriver(capabilities=None):
    """Get remote webdriver. Configured using environment variables
    to setup where the remote webdriver is. options could be a local
    install or using the setup at saucelabs"""

    if not capabilities:
        capabilities = webdriver.DesiredCapabilities.FIREFOX

    # Saucelabs setup,
    capabilities["build"] = os.environ.get("TRAVIS_BUILD_NUMBER")
    capabilities["tags"] = [os.environ.get("TRAVIS_PYTHON_VERSION"), "CI"]
    capabilities["tunnel-identifier"] = os.environ.get("TRAVIS_JOB_NUMBER")

    username = os.environ.get("SAUCE_USERNAME")
    access_key = os.environ.get("SAUCE_ACCESS_KEY")
    hub_url = os.environ.get('SAUCE_HUBURL', "ondemand.saucelabs.com/wd/hub")

    if username and access_key:
        hub_url = "%s:%s@%s" % (username, access_key, hub_url)

    return webdriver.Remote(desired_capabilities=capabilities,
                            command_executor="http://%s" % hub_url)

class SeleniumTestCase(LiveServerTestCase):
    environment = None

    @classmethod
    def setUpClass(cls):
        if not webdriver:
            print ('No selenium installed')
            #return None
            # raise unittest.SkipTest("No selenium installed")

        # Tests for pull requests from forks do not get the SAUCE_USERNAME
        # exposed because of security considerations. Skip selenium tests for
        # those tests
        if os.environ.get('TRAVIS_BUILD_NUMBER') and \
           not os.environ.get('SAUCE_USERNAME'):
            # raise unittest.SkipTest("Travis with no sauce username, skipping")
            print ('Travis with no sauce username, skipping')
            #return None

        super(SeleniumTestCase, cls).setUpClass()

        cls.drivers = []

        try:
            if 'SELENIUM_REMOTE_TESTS' in os.environ or \
               'TRAVIS_BUILD_NUMBER' in os.environ:
                # Fire up remote webdriver
                remote = get_remote_webdriver()
                cls.drivers.append(remote)
            else:
                # Use the firefox webdriver
                firefox = webdriver.Firefox()
                cls.drivers.append(firefox)
        except WebDriverException as error:
            print ('Exception in running webdriver, skipping')
            #return None
            # raise unittest.SkipTest("Exception in running webdriver, skipping " \
                        #  "selenium tests: %s" % str(error))

        cls.nagios_config = adagios.settings.nagios_config
        cls.environment = adagios.utils.FakeAdagiosEnvironment()
        cls.environment.create_minimal_environment()
        cls.environment.configure_livestatus()
        cls.environment.update_adagios_global_variables()
        cls.environment.start()
        cls.livestatus = cls.environment.get_livestatus()

    @classmethod
    def tearDownClass(cls):
        cls.environment.terminate()
        for driver in cls.drivers:
            driver.quit()
        super(SeleniumTestCase, cls).tearDownClass()




# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from django.test import TestCase
from django.test.client import Client

import pynag.Parsers

import tempfile
import os
from adagios.contrib import get_template_name
import pynag.Utils


class ContribTests(TestCase):
    def setUp(self):
        base_path = tempfile.mkdtemp()
        self.base_path = base_path

    def tearDown(self):
        command = ['rm', '-rf', self.base_path]
        pynag.Utils.runCommand(command=command, shell=False)

    def testGetTemplateFilename(self):
        base_path = self.base_path

        file1 = base_path + '/file1'
        dir1 = base_path + '/dir1'
        file2 = dir1 + '/file2'

        open(file1, 'w').write('this is file1')
        os.mkdir(dir1)
        open(file2, 'w').write('this is file2')

        self.assertEqual(file1, get_template_name(base_path, 'file1'))
        self.assertEqual(file2, get_template_name(base_path, 'dir1', 'file2'))
        self.assertEqual(file2, get_template_name(base_path, 'dir1', 'file2', 'unneeded_argument'))

        # Try to return a filename that is outside base_path
        exception1 = lambda: get_template_name(base_path, '/etc/passwd')
        self.assertRaises(Exception, exception1)

        # Try to return a filename that is outside base_path
        exception2 = lambda: get_template_name(base_path, '/etc/', 'passwd')
        self.assertRaises(Exception, exception2)

        # Try to return a filename that is outside base_path
        exception3 = lambda: get_template_name(base_path, '..', 'passwd')
        self.assertRaises(Exception, exception3)








# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.utils.encoding import smart_str
from django import forms

class AdagiosForm(forms.Form):
    """ Base class for all forms in this module. Forms that use pynag in any way should inherit from this one.
    """
    def clean(self):
        cleaned_data = {}
        tmp = super(AdagiosForm, self).clean()
        for k,v in tmp.items():
            if isinstance(k, (unicode)):
                k = smart_str(k)
            if isinstance(v, (unicode)):
                v = smart_str(v)
            cleaned_data[k] = v
        return cleaned_data



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import url, patterns

urlpatterns = patterns('adagios',
                        url(r'^/?$', 'status.views.status_index'),
                        url(r'^/acknowledgements/?$', 'status.views.acknowledgement_list'),
                        url(r'^/error/?$', 'status.views.error_page'),
                        url(r'^/comments/?$', 'status.views.comment_list'),
                        url(r'^/contacts/?$', 'status.views.contact_list'),
                        url(r'^/contactgroups/?$', 'status.views.contactgroups'),
                        url(r'^/dashboard/?$', 'status.views.dashboard'),
                        url(r'^/detail/?$', 'status.views.detail'),
                        url(r'^/downtimes/?$', 'status.views.downtime_list'),
                        url(r'^/hostgroups/?$', 'status.views.status_hostgroups'),
                        url(r'^/hosts/?$', 'status.views.hosts'),
                        url(r'^/log/?$', 'status.views.log'),
                        url(r'^/map/?', 'status.views.map_view'),
                        url(r'^/parents/?$', 'status.views.network_parents'),
                        url(r'^/perfdata/?$', 'status.views.perfdata'),
                        url(r'^/perfdata2/?$', 'status.views.perfdata2'),
                        url(r'^/problems/?$', 'status.views.problems'),
                        url(r'^/servicegroups/?$', 'status.views.status_servicegroups'),
                        url(r'^/services/?$', 'status.views.services'),
                        url(r'^/state_history/?$', 'status.views.state_history'),
                        url(r'^/backends/?$', 'status.views.backends'),



                        # Misc snippets
                        url(r'^/snippets/log/?$', 'status.views.snippets_log'),
                        url(r'^/snippets/services/?$', 'status.views.snippets_services'),
                        url(r'^/snippets/hosts/?$', 'status.views.snippets_hosts'),

                        # Misc tests
                        url(r'^/test/services/?$', 'status.views.services_js'),
                        url(r'^/test/status_dt/?$', 'status.views.status_dt'),
                        url(r'^/test/livestatus/?$', 'status.views.test_livestatus'),

                        # Deprecated as of 2013-03-23
                        url(r'^/contacts/(?P<contact_name>.+)/?$', 'status.views.contact_detail'),
                        url(r'^/hostgroups/(?P<hostgroup_name>.+)/?$', 'status.views.status_hostgroup'),
                        url(r'^/contactgroups/(?P<contactgroup_name>.+)/?$', 'status.views.contactgroup_detail'),
                        url(r'^/servicegroups/(?P<servicegroup_name>.+)/?$', 'status.views.servicegroup_detail'),
                        url(r'^/services_old/?$', 'status.views.status'),


                        )



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This file demonstrates two different styles of tests (one doctest and one
unittest). These will both pass when you run "manage.py test".

Replace these with more appropriate tests for your application.
"""

from django.test import TestCase
from django.test.client import Client
from django.utils.translation import ugettext as _
import json

import adagios.utils


class LiveStatusTestCase(TestCase):

    def setUp(self):
        self.environment = adagios.utils.get_test_environment()
        self.environment.start()
        self.addCleanup(self.environment.terminate)

    def testPageLoad(self):
        """ Smoke Test for various rest modules """
        self.loadPage('/rest')
        self.loadPage('/rest/status/')
        self.loadPage('/rest/pynag/')
        self.loadPage('/rest/adagios/')
        self.loadPage('/rest/status.js')
        self.loadPage('/rest/pynag.js')
        self.loadPage('/rest/adagios.js')

    def testDnsLookup(self):
        """ Test the DNS lookup rest call
        """
        path = "/rest/pynag/json/dnslookup"
        data = {'host_name': 'localhost'}
        try:
            c = Client()
            response = c.post(path=path, data=data)
            json_data = json.loads(response.content)
            self.assertEqual(response.status_code, 200, _("Expected status code 200 for page %s") % path)
            self.assertEqual(True, 'addresslist' in json_data, _("Expected 'addresslist' to appear in response"))
        except KeyError, e:
            self.assertEqual(True, _("Unhandled exception while loading %(path)s: %(exc)s") % {'path': path, 'exc': e})

    def testGetAllHostsViaJSON(self):
        """Test fetching hosts via json"""
        path = "/rest/status/json/hosts"
        data = {'fields': 'host_name'}
        c = Client()
        response = c.post(path=path, data=data)
        json_data = json.loads(response.content)
        self.assertEqual(response.status_code, 200, _("Expected status code 200 for page %s") % path)
        self.assertEqual(['ok_host'], [x['name'] for x in json_data])
        self.assertEqual(['name', 'backend'], json_data[0].keys())

    def loadPage(self, url):
        """ Load one specific page, and assert if return code is not 200 """
        try:
            c = Client()
            response = c.get(url)
            self.assertEqual(response.status_code, 200, _("Expected status code 200 for page %s") % url)
        except Exception, e:
            self.assertEqual(True, _("Unhandled exception while loading %(url)s: %(exc)s") % {'url': url, 'exc': e})



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import url, patterns, include
from adagios import settings
from django.views.static import serve

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns(
    '',
    # Example:
    url(r'^$', 'adagios.views.index', name="home"),
    url(r'^403', 'adagios.views.http_403'),
    url(r'^objectbrowser', include('adagios.objectbrowser.urls')),
    url(r'^status', include('adagios.status.urls')),
    url(r'^bi', include('adagios.bi.urls')),
    url(r'^misc', include('adagios.misc.urls')),
    url(r'^pnp', include('adagios.pnp.urls')),
    url(r'^media(?P<path>.*)$',         serve, {'document_root': settings.STATIC_ROOT }),
    url(r'^rest', include('adagios.rest.urls')),
    url(r'^contrib', include('adagios.contrib.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    # (r'^admin/', include(admin.site.urls)),

    # Internationalization
    url(r'^jsi18n/$', 'django.views.i18n.javascript_catalog'),
)

if settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.STATIC_ROOT}, name="media"),
    )
        

# from django.contrib.staticfiles.urls import staticfiles_urlpatterns
#urlpatterns += staticfiles_urlpatterns()




# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from django.test.client import Client
from django.utils.translation import ugettext as _

import pynag.Parsers
import os
from django.test.client import RequestFactory
from django.test import LiveServerTestCase
import adagios.status
import adagios.status.utils
import adagios.status.graphite
import adagios.settings
import adagios.utils
import simplejson as json
import adagios.seleniumtests
from mock import patch
import adagios.status.rest

try:
    from selenium.webdriver.common.by import By
except ImportError:
    pass


class LiveStatusTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.nagios_config = adagios.settings.nagios_config
        cls.environment = adagios.utils.get_test_environment()
        cls.environment.start()

        cls.livestatus = cls.environment.get_livestatus()

        cls.factory = RequestFactory()

    @classmethod
    def tearDownClass(cls):
        cls.environment.terminate()

    def testLivestatusConnectivity(self):
        requests = self.livestatus.query('GET status', 'Columns: requests')
        self.assertEqual(
            1, len(requests), _("Could not get status.requests from livestatus"))

    def testLivestatusConfigured(self):
        config = pynag.Parsers.config(cfg_file=self.nagios_config)
        config.parse_maincfg()
        for k, v in config.maincfg_values:
            if k == "broker_module" and v.find('livestatus') > 1:
                tmp = v.split()
                self.assertFalse(
                    len(tmp) < 2, _(' We think livestatus is incorrectly configured. In nagios.cfg it looks like this: %s') % v)
                module_file = tmp[0]
                socket_file = tmp[1]
                self.assertTrue(
                    os.path.exists(module_file), _(' Livestatus Broker module not found at "%s". Is nagios correctly configured?') % module_file)
                self.assertTrue(
                    os.path.exists(socket_file), _(' Livestatus socket file was not found (%s). Make sure nagios is running and that livestatus module is loaded') % socket_file)
                return
        self.assertTrue(
            False, _('Nagios Broker module not found. Is livestatus installed and configured?'))

    def testPageLoad(self):
        """ Loads a bunch of status pages, looking for a crash """
        self.loadPage('/status/')
        self.loadPage('/status/hosts')
        self.loadPage('/status/services')
        self.loadPage('/status/contacts')
        self.loadPage('/status/parents')
        self.loadPage('/status/state_history')
        self.loadPage('/status/log')
        self.loadPage('/status/comments')
        self.loadPage('/status/downtimes')
        self.loadPage('/status/hostgroups')
        self.loadPage('/status/servicegroups')
        self.loadPage('/status/map')
        self.loadPage('/status/dashboard')

    def test_status_detail(self):
        """ Tests for /status/detail """
        tmp = self.loadPage('/status/detail?contact_name=nagiosadmin')
        self.assertTrue('nagiosadmin belongs to the following' in tmp.content)

        tmp = self.loadPage('/status/detail?host_name=ok_host')
        self.assertTrue('ok_host' in tmp.content)

        tmp = self.loadPage('/status/detail?host_name=ok_host&service_description=ok%20service%201')
        self.assertTrue('ok_host' in tmp.content)

        tmp = self.loadPage('/status/detail?contactgroup_name=admins')
        self.assertTrue('nagiosadmin' in tmp.content)


    def testStateHistory(self):
        request = self.factory.get('/status/state_history')
        adagios.status.views.state_history(request)

    def loadPage(self, url, expected_status_code=200):
        """ Load one specific page, and assert if return code is not 200 """
        c = Client()
        response = c.get(url)
        self.assertEqual(response.status_code, expected_status_code, _("Expected status code %(code)s for page %(url)s") % {'code': expected_status_code, 'url': url})
        return response

    def testSubmitCommand(self):
        """ Test adagios.rest.status.submit_check_results
        """
        c = Client()
        data = {}
        data['host_name'] = 'adagios test host'
        data['service_description'] = 'nonexistant'
        data['status_code'] = "0"
        data['plugin_output'] = 'test message'
        data['performance_data'] = ''
        response = c.post('/rest/status/json/submit_check_result', data=data)
        self.assertEqual(200, response.status_code)

    def test_rest_top_alert_producers(self):
        """ Test adagios.rest.status.top_alert_producers
        """
        c = Client()
        data = {}
        data['limit'] = '1'
        response = c.post('/rest/status/json/top_alert_producers', data=data)
        self.assertEqual(200, response.status_code)

        json_reply = json.loads(response.content)
        self.assertFalse(len(json_reply) > 1, "Too many objects returned" \
                         " limit is 1, json:\n" + str(json_reply))

class Graphite(TestCase):
    def test__get_graphite_url(self):
        """ Smoketest for  adagios.status.graphite._get_graphite_url() """
        base = "http://localhost/graphite"
        host = "localhost"
        service = "Ping"
        metric = "packetloss"
        from_ = "-1d"
        parameters = locals()
        parameters.pop('self', None)
        result = adagios.status.graphite._get_graphite_url(**parameters)
        self.assertTrue(result.startswith(base))
        self.assertTrue(host in result)
        self.assertTrue(service in result)
        self.assertTrue(metric in result)

    def test_get(self):
        """ Smoketest for adagios.status.graphite.get() """
        base = "http://localhost/graphite"
        host = "localhost"
        service = "Ping"
        metrics = ["packetloss", "rta"]
        units = [("test", "test", "-1d")]
        parameters = locals()
        parameters.pop('self', None)
        result = adagios.status.graphite.get(**parameters)
        self.assertTrue(result)
        self.assertTrue(len(result) == 1)
        self.assertTrue('rta' in result[0]['metrics'])
        self.assertTrue('packetloss' in result[0]['metrics'])


class UtilsTest(TestCase):
    """Tests for adagios.status.utils"""
    def setUp(self):
        self.host_query = pynag.Parsers.LivestatusQuery('GET hosts')
        self.service_query = pynag.Parsers.LivestatusQuery('GET services')

    def test_search_multiple_attributes_multiple_attributes(self):
        attributes = ['host_name', 'address']
        adagios.status.utils._search_multiple_attributes(self.host_query, attributes, 'test')
        expected_query = 'GET hosts\nFilter: host_name = test\nFilter: address = test\nOr: 2\n\n'
        self.assertEqual(expected_query, self.host_query.get_query())

    def test_search_multiple_attributes_one_attribute(self):
        attributes = ['host_name']
        adagios.status.utils._search_multiple_attributes(self.host_query, attributes, 'test')
        expected_query = 'GET hosts\nFilter: host_name = test\n\n'
        self.assertEqual(expected_query, self.host_query.get_query())

    def test_search_multiple_attributes_empty(self):
        adagios.status.utils._search_multiple_attributes(self.host_query, [], 'test')
        expected_query = 'GET hosts\n\n'
        self.assertEqual(expected_query, self.host_query.get_query())

    def test__process_querystring_for_host_empty(self):
        query = adagios.status.utils._process_querystring_for_host()
        self.assertEqual(self.host_query, query)

    def test__process_querystring_for_host_one_kwarg(self):
        query = adagios.status.utils._process_querystring_for_host(state="1")
        self.host_query.add_filter('state', '1')
        self.assertEqual(self.host_query, query)

    def test__process_querystring_for_host_two_kwargs(self):
        query = adagios.status.utils._process_querystring_for_host(state="1", host_name="localhost")
        self.host_query.add_filters(state=1, host_name="localhost")
        self.assertEqual(self.host_query, query)

    def test__process_querystring_for_host_generic_search(self):
        search = "test"
        query = adagios.status.utils._process_querystring_for_host(q=search)
        self.host_query.add_filter('name__contains', search)
        self.host_query.add_filter('address__contains', search)
        self.host_query.add_filter('plugin_output__contains', search)
        self.host_query.add_filter('alias__contains', search)
        self.host_query.add_or_statement(4)
        self.assertEqual(self.host_query, query)

    def test__process_querystring_for_host_unhandled(self):
        query = adagios.status.utils._process_querystring_for_host(unhandled=True)
        self.host_query.add_filter('scheduled_downtime_depth', 0)
        self.host_query.add_filter('state', 1)
        self.host_query.add_filter('acknowledged', 0)
        self.assertEqual(self.host_query, query)

    def test__process_querystring_for_host_downtime_on(self):
        query = adagios.status.utils._process_querystring_for_host(in_scheduled_downtime="1")
        self.host_query.add_filter('scheduled_downtime_depth__gt', 0)
        self.assertEqual(self.host_query, query)

    def test__process_querystring_for_host_downtime_off(self):
        query = adagios.status.utils._process_querystring_for_host(in_scheduled_downtime="0")
        self.host_query.add_filter('scheduled_downtime_depth', 0)
        self.assertEqual(self.host_query, query)

    def test__process_querystring_for_service_empty(self):
        query = adagios.status.utils._process_querystring_for_service()
        self.assertEqual(self.service_query, query)

    def test__process_querystring_for_service_one_kwarg(self):
        query = adagios.status.utils._process_querystring_for_service(state="1")
        self.service_query.add_filter('state', '1')
        self.assertEqual(self.service_query, query)

    def test__process_querystring_for_service_two_kwargs(self):
        query = adagios.status.utils._process_querystring_for_service(state="1", description="localservice")
        self.service_query.add_filters(state=1, description="localservice")
        self.assertEqual(self.service_query, query)

    def test__process_querystring_for_service_generic_search(self):
        search = "test"
        query = adagios.status.utils._process_querystring_for_service(q=search)
        self.service_query.add_filter('host_name__contains', search)
        self.service_query.add_filter('description__contains', search)
        self.service_query.add_filter('plugin_output__contains', search)
        self.service_query.add_filter('host_address__contains', search)
        self.service_query.add_or_statement(4)
        self.assertEqual(self.service_query, query)

    def test__process_querystring_for_service_unhandled(self):
        query = adagios.status.utils._process_querystring_for_service(unhandled=True)
        self.service_query.add_filter('host_scheduled_downtime_depth', 0)
        self.service_query.add_filter('host_state', 0)
        self.service_query.add_filter('scheduled_downtime_depth', 0)
        self.service_query.add_filter('acknowledged', 0)
        self.service_query.add_filter('state__isnot', 0)
        self.service_query.add_filter('host_acknowledged', 0)
        self.assertEqual(self.service_query, query)

    def test__process_querystring_for_service_downtime_on(self):
        query = adagios.status.utils._process_querystring_for_service(in_scheduled_downtime="1")
        self.service_query.add_filter('scheduled_downtime_depth__gt', 0)
        self.assertEqual(self.service_query, query)

    def test__process_querystring_for_service_downtime_off(self):
        query = adagios.status.utils._process_querystring_for_service(in_scheduled_downtime="0")
        self.service_query.add_filter('scheduled_downtime_depth', 0)
        self.assertEqual(self.service_query, query)


class SeleniumStatusTestCase(adagios.seleniumtests.SeleniumTestCase):
    def test_network_parents(self):
        """Status Overview, Network Parents should show an integer"""
        for driver in self.drivers:
            driver.get(self.live_server_url + "/status")

            # Second link is Network Parents in overview
            self.assertEqual(driver.find_elements(By.XPATH,
                "//a[@href='/status/parents']")[1].text.isdigit(), True)

    def test_services_select_all(self):
        """Loads services list and tries to select everything

        Flow:
            Load http://<url>/status/services
            Click select all
            Look for statustable rows
            Assert that all rows are checked"""

        for driver in self.drivers:
            driver.get(self.live_server_url + "/status/services")

            driver.find_element_by_xpath("//input[@class='select_many']").click()
            driver.find_element_by_xpath("//a[@class='select_all']").click()

            # Get all statustable rows
            status_table_rows = driver.find_element_by_xpath(
                "//table[contains(@class, 'statustable')]"
            ).find_elements(By.XPATH, "//tbody/tr[contains(@class, 'mainrow')]")

            # Sub-select non-selected
            for row in status_table_rows:
                self.assertTrue('row_selected' in row.get_attribute('class'),
                                "Non selected row found after selecting all: " + \
                                row.text)

    def test_status_overview_top_alert_producers(self):
        """Check the top alert producers part of status overview"""
        for driver in self.drivers:
            driver.get(self.live_server_url + "/status")

            top_alert_table_rows = driver.find_elements(By.XPATH,
                "//table[@id='top_alert_producers']/tbody/tr"
            )

            count = 0
            for row in top_alert_table_rows:
                if 'display' not in row.get_attribute('style'):
                    count += 1

            self.assertTrue(count <= 3, "Top alert producers returns too many rows")

class RestTests(TestCase):
    def test_reschedule_raises_on_float_string_check_time(self):
        """reschedule should raise on string float"""
        self.assertRaises(ValueError, adagios.status.rest.reschedule,
                          request={},
                          host_name='localhost',
                          check_time='1.1')

    @patch('pynag.Control.Command.schedule_forced_host_check')
    def test_reschedule_calls_host_check_with_int_check_time(self,
                                                             mock_host_check):
        """reschedule converts to int before calling schedule_forced_host_check
        """
        mock_host_check.return_value = None
        adagios.status.rest.reschedule(request={},
                                       host_name='localhost',
                                       check_time=1.1)
        mock_host_check.assert_called_once_with(host_name='localhost',
                                                check_time=1)

    @patch('time.time')
    @patch('pynag.Control.Command.schedule_forced_host_check')
    def test_reschedule_uses_current_time_with_no_check_time(
            self, mock_host_check, mock_time):
        """reschedule uses current time as int when no check_time specified"""
        mock_host_check.return_value = None
        mock_time.return_value = 7357.7357
        adagios.status.rest.reschedule(request={},
                                       host_name='localhost')
        mock_host_check.assert_called_once_with(host_name='localhost',
                                                check_time=7357)

    @patch('time.time')
    @patch('pynag.Control.Command.schedule_forced_host_check')
    def test_reschedule_calls_host_check_with_current_time_when_empty_string(
            self,
            mock_host_check,
            mock_time):
        """reschedule called with empty string sets check_time to current"""
        mock_host_check.return_value = None
        mock_time.return_value = 7357.7357
        adagios.status.rest.reschedule(request={},
                                       host_name='localhost',
                                       check_time='')
        mock_host_check.assert_called_once_with(host_name='localhost',
                                                check_time=7357)



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

import unittest
#from django.test import TestCase
from django.test.client import Client
from django.utils.translation import ugettext as _

import pynag.Parsers
from adagios.settings import nagios_config
from adagios.pnp import functions


class PNP4NagiosTestCase(unittest.TestCase):
    
    @unittest.skip("...skipping")
    def testPnpIsConfigured(self):
        config = pynag.Parsers.config()
        config.parse_maincfg()
        for k, v in config.maincfg_values:
            if k == "broker_module" and v.find('npcd') > 1:
                tmp = v.split()
                self.assertFalse(
                    len(tmp) < 2, _('We think pnp4nagios broker module is incorrectly configured. In nagios.cfg it looks like this: %s') % v)
                module_file = tmp.pop(0)
                self.assertTrue(
                    os.path.exists(module_file), _('npcd broker_module module not found at "%s". Is nagios correctly configured?') % module_file)

                config_file = None
                for i in tmp:
                    if i.startswith('config_file='):
                        config_file = i.split('=', 1)[1]
                        break
                self.assertIsNotNone(
                    config_file, _("npcd broker module has no config_file= argument. Is pnp4nagios correctly configured?"))
                self.assertTrue(
                    os.path.exists(config_file), _('PNP4nagios config file was not found (%s).') % config_file)
                return
        self.assertTrue(
            False, _('Nagios Broker module not found. Is pnp4nagios installed and configured?'))

    @unittest.skip("...skipping")
    def testGetJson(self):
        result = functions.run_pnp('json')
        self.assertGreaterEqual(
            len(result), 0, msg=_("Tried to get json from pnp4nagios but result was improper"))

    @unittest.skip("...skipping")
    def testPageLoad(self):
        c = Client()
        response = c.get('/pnp/json')
        self.assertEqual(response.status_code, 200)



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models

# Create your models here.


class Attribute(models.Model):

    """This class stores info on how attributes are viewed in django"""
    attribute_name = models.CharField(max_length=200)
    attribute_friendlyname = models.CharField(max_length=200)
    attribute_type = models.CharField(max_length=200)



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Matthieu Caneill <matthieu.caneill@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import re
import adagios.settings

ILLEGAL_CHAR = re.compile(r'[^\w-]')


def _get_graphite_url(base, host, service, metric, from_):
    """ Constructs an URL for Graphite.

    Args:
      - base (str): base URL for Graphite access
      - host (str): hostname
      - service (str): service, e.g. HTTP
      - metric (str): metric, e.g. size, time
      - from_ (str): Graphite time period

    Returns: str
    """
    host_ = _compliant_name(host)
    service_ = _compliant_name(service)
    metric_ = _compliant_name(metric)
    base = base.rstrip('/')
    title = adagios.settings.graphite_title.format(**locals())

    url = "{base}/render?" + adagios.settings.graphite_querystring
    url = url.format(**locals())
    return url


def _compliant_name(name):
    """ Makes the necessary replacements for Graphite. """
    if name == '_HOST_':
        return '__HOST__'
    name = ILLEGAL_CHAR.sub('_', name)
    return name


def get(base, host, service, metrics, units):
    """ Returns a data structure containg URLs for Graphite.

    The structure looks like:
    [{'name': 'One day',
      'css_id' : 'day',
      'metrics': {'size': 'http://url-of-size-metric',
                  'time': 'http://url-of-time-metric'}
     },
     {...}]

    Args:
      - base (str): base URL for Graphite access
      - host (str): hostname
      - service (str): service, e.g. HTTP
      - metrics (list): list of metrics, e.g. ["size", "time"]
      - units (list): a list of <name,css_id,unit>,
        see adagios.settings.GRAPHITE_PERIODS

    Returns: list
    """
    graphs = []

    for name, css_id, unit in units:
        m = {}
        for metric in metrics:
            m[metric] = _get_graphite_url(base, host, service, metric, unit)
        graph = dict(name=name, css_id=css_id, metrics=m)
        graphs.append(graph)

    return graphs



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2010, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.context_processors import csrf
from django.forms.formsets import BaseFormSet
from django.shortcuts import render_to_response
from django.shortcuts import render
from django.utils.translation import ugettext as _

from django.shortcuts import HttpResponse
from django.template import RequestContext
from adagios.misc import forms
import os
import mimetypes

import pynag.Model
import pynag.Utils
import adagios.daemon
import pynag.Model.EventHandlers
import pynag.Utils
import os.path
from time import mktime, sleep
from datetime import datetime
from os.path import dirname
from subprocess import Popen, PIPE

import adagios.settings
import adagios.objectbrowser
from adagios import __version__
import adagios.status.utils
from adagios import userdata

from collections import defaultdict
from adagios.views import adagios_decorator, error_page

state = defaultdict(lambda: "unknown")
state[0] = "ok"
state[1] = "warning"
state[2] = "critical"

@adagios_decorator
def index(request):
    c = {}
    c['nagios_cfg'] = pynag.Model.config.cfg_file
    c['version'] = __version__
    return render_to_response('frontpage.html', c, context_instance=RequestContext(request))

@adagios_decorator
def settings(request):
    c = {}
    c.update(csrf(request))
    c['messages'] = m = []
    c['errors'] = e = []
    if request.method == 'GET':
        form = forms.AdagiosSettingsForm(initial=request.GET)
        form.is_valid()
    elif request.method == 'POST':
        form = forms.AdagiosSettingsForm(data=request.POST)
        if form.is_valid():
            try:
                form.save()
                m.append(_("%s successfully saved.") % form.adagios_configfile)
            except IOError, exc:
                e.append(exc)
    else:
        raise Exception(_("We only support methods GET or POST"))
    c['form'] = form
    return render_to_response('settings.html', c, context_instance=RequestContext(request))


@adagios_decorator
def nagios(request):
    return iframe(request, adagios.settings.nagios_url)

@adagios_decorator
def iframe(request, url=None):
    if not url:
        url = request.GET.get('url', None)
    return render_to_response('iframe.html', locals(), context_instance=RequestContext(request))


@adagios_decorator
def gitlog(request):
    """ View that displays a nice log of previous git commits in dirname(config.cfg_file) """
    c = {}
    c.update(csrf(request))
    c['messages'] = m = []
    c['errors'] = []

    # Get information about the committer
    author_name = request.META.get('REMOTE_USER', 'anonymous')
    try:
        contact = pynag.Model.Contact.objects.get_by_shortname(author_name)
        author_email = contact.email or None
    except Exception:
        author_email = None
    nagiosdir = dirname(pynag.Model.config.cfg_file or None)
    git = pynag.Utils.GitRepo(
        directory=nagiosdir, author_name=author_name, author_email=author_email)

    c['nagiosdir'] = nagiosdir
    c['commits'] = []
    if request.method == 'POST':

        try:
            if 'git_init' in request.POST:
                git.init()
            elif 'git_commit' in request.POST:
                filelist = []
                commit_message = request.POST.get(
                    'git_commit_message', _("bulk commit by adagios"))
                for i in request.POST:
                    if i.startswith('commit_'):
                        filename = i[len('commit_'):]
                        git.add(filename)
                        filelist.append(filename)
                if len(filelist) == 0:
                    raise Exception(_("No files selected."))
                git.commit(message=commit_message, filelist=filelist)
                m.append(_("%s files successfully commited.") % len(filelist))
        except Exception, e:
            c['errors'].append(e)
    # Check if nagiosdir has a git repo or not
    try:
        c['uncommited_files'] = git.get_uncommited_files()
    except pynag.Model.EventHandlers.EventHandlerError, e:
        if e.errorcode == 128:
            c['no_git_repo_found'] = True

    # Show git history
    try:
        c['commits'] = git.log()

        commit = request.GET.get('show', False)
        if commit != False:
            c['diff'] = git.show(commit)
            difflines = []
            for i in c['diff'].splitlines():
                if i.startswith('---'):
                    tag = 'hide'
                elif i.startswith('+++'):
                    tag = 'hide'
                elif i.startswith('index'):
                    tag = 'hide'
                elif i.startswith('-'):
                    tag = "alert-danger"
                elif i.startswith('+'):
                    tag = "alert-success"
                elif i.startswith('@@'):
                    tag = 'alert-unknown'
                elif i.startswith('diff'):
                    tag = "filename"
                else:
                    continue
                difflines.append({'tag': tag, 'line': i})
            c['difflines'] = difflines
            c['commit_id'] = commit
    except Exception, e:
        c['errors'].append(e)
    return render_to_response('gitlog.html', c, context_instance=RequestContext(request))


@adagios_decorator
def nagios_service(request):
    """ View to restart / reload nagios service """
    c = {}
    c['errors'] = []
    c['messages'] = []

    if request.method == 'GET':
        form = forms.NagiosServiceForm(initial=request.GET)
    else:
        form = forms.NagiosServiceForm(data=request.POST)
        if form.is_valid():
            form.save()
            c['stdout'] = form.stdout
            c['stderr'] = form.stderr
            c['command'] = form.command
            c['exit_code'] = form.exit_code

            for i in form.stdout.splitlines():
                if i.strip().startswith('Error:'):
                    c['errors'].append(i)
    c['form'] = form
    daemon = adagios.daemon.Daemon()
    c['nagios_bin'] = daemon.nagios_bin
    if daemon.running():
        c['status'] = 0
        c['friendly_status'] = "running"
    else:
        c['status'] = 1
        c['friendly_status'] = "not running"
    needs_reload = pynag.Model.config.needs_reload()
    c['needs_reload'] = needs_reload
    return render_to_response('nagios_service.html', c, context_instance=RequestContext(request))


@adagios_decorator
def pnp4nagios(request):
    """ View to handle integration with pnp4nagios """
    c = {}
    c['errors'] = e = []
    c['messages'] = m = []

    c['broker_module'] = forms.PNPBrokerModuleForm(initial=request.GET)
    c['templates_form'] = forms.PNPTemplatesForm(initial=request.GET)
    c['action_url'] = forms.PNPActionUrlForm(initial=request.GET)
    c['pnp_templates'] = forms.PNPTemplatesForm(initial=request.GET)

    try:
        c['npcd_config'] = forms.PNPConfigForm(initial=request.GET)
    except Exception, e:
        c['errors'].append(e)
    #c['interesting_objects'] = form.interesting_objects
    if request.method == 'POST' and 'save_broker_module' in request.POST:
        c['broker_module'] = broker_form = forms.PNPBrokerModuleForm(
            data=request.POST)
        if broker_form.is_valid():
            broker_form.save()
            m.append(_("Broker Module updated in nagios.cfg"))
    elif request.method == 'POST' and 'save_action_url' in request.POST:
        c['action_url'] = forms.PNPActionUrlForm(data=request.POST)
        if c['action_url'].is_valid():
            c['action_url'].save()
            m.append(_('Action_url updated for %s services') %
                     c['action_url'].total_services)
            if c['action_url'].error_services > 0:
                e.append(
                    _("%s services could not be updated (check permissions?)") %
                    c['action_url'].error_services)
    elif request.method == 'POST' and 'save_npcd_config' in request.POST:
        c['npcd_config'] = forms.PNPConfigForm(data=request.POST)
        if c['npcd_config'].is_valid():
            c['npcd_config'].save()
            m.append(_("npcd.cfg updated"))

    return render_to_response('pnp4nagios.html', c, context_instance=RequestContext(request))


@adagios_decorator
def edit_file(request, filename):
    """ This view gives raw read/write access to a given filename.

     Please be so kind as not to give direct url access to this function, because it will allow
     Editing of any file the webserver has access to.
    """
    c = {}
    c['messages'] = []
    c['errors'] = []
    try:
        c['form'] = forms.EditFileForm(filename=filename, initial=request.GET)
        c['filename'] = filename
        if request.method == 'POST':
            c['form'] = forms.EditFileForm(
                filename=filename, data=request.POST)
            if c['form'].is_valid():
                c['form'].save()
    except Exception, e:
        c['errors'].append(e)
    return render_to_response('editfile.html', c, context_instance=RequestContext(request))


@adagios_decorator
def edit_nagios_cfg(request):
    """ Allows raw editing of nagios.cfg configfile
    """
    return edit_file(request, filename=adagios.settings.nagios_config)


@adagios_decorator
def pnp4nagios_edit_template(request, filename):
    """ Allows raw editing of a pnp4nagios template.

     Will throw security exception if filename is not a pnp4nagios template
    """

    form = forms.PNPTemplatesForm(initial=request.GET)
    if filename in form.templates:
        return edit_file(request, filename=filename)
    else:
        raise Exception(
            _("Security violation. You are not allowed to edit %s") % filename)


@adagios_decorator
def icons(request, image_name=None):
    """ Use this view to see nagios icons/logos
    """
    c = {}
    c['messages'] = []
    c['errors'] = []
    image_path = '/usr/share/nagios3/htdocs/images/logos/'
    filenames = []
    for root, subfolders, files in os.walk(image_path):
        for filename in files:
            filenames.append(os.path.join(root, filename))
    # Cut image_path out of every filename
    filenames = map(lambda x: x[len(image_path):], filenames)

    # Filter out those silly .gd2 files that don't display inside a browser
    filenames = filter(lambda x: not x.lower().endswith('.gd2'), filenames)

    filenames.sort()
    if not image_name:
        # Return a list of images
        c['images'] = filenames
        return render_to_response('icons.html', c, context_instance=RequestContext(request))
    else:
        if image_name in filenames:
            file_extension = image_name.split('.').pop()
            mime_type = mimetypes.types_map.get(file_extension)
            fsock = open("%s/%s" % (image_path, image_name,))
            return HttpResponse(fsock, content_type=mime_type)
        else:
            raise Exception(_("Not allowed to see this image"))


@adagios_decorator
def mail(request):
    """ Send a notification email to one or more contacts regarding hosts or services """
    c = {}
    c['messages'] = []
    c['errors'] = []
    c.update(csrf(request))
    c['http_referer'] = request.META.get("HTTP_REFERER")
    c['http_origin'] = request.META.get("HTTP_ORIGIN")
    remote_user = request.META.get('REMOTE_USER', 'anonymous adagios user')
    hosts = []
    services = []
    if request.method == 'GET':
        c['form'] = forms.SendEmailForm(remote_user, initial=request.GET)
        hosts = request.GET.getlist('host') or request.GET.getlist('host[]')
        services = request.GET.getlist(
            'service') or request.GET.getlist('service[]')
        if not services and not hosts:
            c['form'].services = adagios.status.utils.get_services(request, host_name='localhost')
    elif request.method == 'POST':
        c['form'] = forms.SendEmailForm(remote_user, data=request.POST)
        services = request.POST.getlist('service') or request.POST.getlist('service[]')
        hosts = request.POST.getlist('host') or request.POST.getlist('host[]')
        c['acknowledged_or_not'] = request.POST.get('acknowledge_all_problems') == 'true'

    for host_name in hosts:
        host_object = adagios.status.utils.get_hosts(request, host_name=host_name)
        if not host_object:
            c['errors'].append(
                _("Host %s not found. Maybe a typo or you do not have access to it.") % host_name
            )
            continue
        for item in host_object:
            item['host_name'] = item['name']
            item['description'] = _("Host Status")
            c['form'].status_objects.append(item)
            c['form'].hosts.append(item)

    for i in services:
        try:
            host_name, service_description = i.split('/', 1)
            service = adagios.status.utils.get_services(
                request, host_name=host_name, service_description=service_description)
            if not service:
                c['errors'].append(
                    _('Service "%s"" not found. Maybe a typo or you do not have access to it ?') % i)
            for x in service:
                c['form'].status_objects.append(x)
                c['form'].services.append(x)
        except AttributeError, e:
            c['errors'].append(_("AttributeError for '%(i)s': %(e)s") % {'i': i, 'e': e})
        except KeyError, e:
            c['errors'].append(_("Error adding service '%(i)s': %(e)s") % {'i': i, 'e': e})

    c['services'] = c['form'].services
    c['hosts'] = c['form'].hosts
    c['status_objects'] = c['form'].status_objects
    c['form'].html_content = render(
        request, "snippets/misc_mail_objectlist.html", c).content
    if request.method == 'POST' and c['form'].is_valid():
        c['form'].save()
    return render_to_response('misc_mail.html', c, context_instance=RequestContext(request))



@adagios_decorator
def test(request):
    """ Generic test view, use this as a sandbox if you like
    """
    c = {}
    c['messages'] = []
    c.update(csrf(request))
    # Get some test data

    if request.method == 'POST':
        c['form'] = forms.PluginOutputForm(data=request.POST)
        if c['form'].is_valid():
            c['form'].parse()
    else:
        c['form'] = forms.PluginOutputForm(initial=request.GET)

    return render_to_response('test.html', c, context_instance=RequestContext(request))


@adagios_decorator
def paste(request):
    """ Generic test view, use this as a sandbox if you like
    """
    c = {}
    c['messages'] = []
    c.update(csrf(request))
    # Get some test data

    if request.method == 'POST':
        c['form'] = forms.PasteForm(data=request.POST)
        if c['form'].is_valid():
            c['form'].parse()
    else:
        c['form'] = forms.PasteForm(initial=request.GET)

    return render_to_response('test2.html', c, context_instance=RequestContext(request))

@adagios_decorator
def preferences(request):
    c = {}
    c['messages'] = []
    c.update(csrf(request))
    
    user = userdata.User(request)
    
    if request.method == 'POST':
        c['form'] = forms.UserdataForm(data=request.POST)
        if c['form'].is_valid():
            for k, v in c['form'].cleaned_data.iteritems():
                user.set_pref(k, v)
            user.save() # will save in json and trigger the hooks
            c['messages'].append(_('Preferences have been saved.'))
    else:
        c['form'] = forms.UserdataForm(initial=user.to_dict())

    return render_to_response('userdata.html', c, context_instance=RequestContext(request))



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2010, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.context_processors import csrf
from django.shortcuts import render_to_response
from django.shortcuts import HttpResponse
from adagios.pnp.functions import run_pnp
from adagios.views import adagios_decorator
import json


@adagios_decorator
def pnp(request, pnp_command='image'):
    c = {}
    c['messages'] = []
    c['errors'] = []
    result = run_pnp(pnp_command, **request.GET)
    content_type = "text"
    if pnp_command == 'image':
        content_type = "image/png"
    elif pnp_command == 'json':
        content_type = "application/json"
    return HttpResponse(result, content_type=content_type)



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2010, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms

from django.core.mail import send_mail
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _

import os.path
from adagios import settings
import adagios.utils
from pynag import Model
from django.core.mail import EmailMultiAlternatives
import pynag.Parsers
import pynag.Control.Command
import adagios.daemon


TOPIC_CHOICES = (
    ('general', _('General Suggestion')),
    ('bug', _('I think i have found a bug')),
    ('suggestion', _('I have a particular task in mind that i would like to do with Adagios')),
    ('easier', _('I have an idea how make a certain task easier to do')),
)

pnp_loglevel_choices = [
    ('0', _('0 - Only Errors')),
    ('1', _('1 - Little logging')),
    ('2', _('2 - Log Everything')),
    ('-1', _('-1 Debug mode (log all and slower processing'))
]
pnp_log_type_choices = [('syslog', 'syslog'), ('file', 'file')]

COMMAND_CHOICES = [('reload', 'reload'), ('status', 'status'),
                   ('restart', 'restart'), ('stop', 'stop'), ('start', 'start')]


initial_paste = """
define service {
    host_name  host01.example.com
    service_description http://host01.example.com
    use     template-http
}

define service {
    name        template-http
    check_command   okc-check_http
}
"""

class ContactUsForm(forms.Form):
    topic = forms.ChoiceField(choices=TOPIC_CHOICES)
    sender = forms.CharField(
        required=False,
        help_text=_("Optional email address if you want feedback from us"),
    )
    message = forms.CharField(
        widget=forms.widgets.Textarea(
            attrs={'rows': 15, 'cols': 40}),
        help_text=_("See below for examples of good suggestions"),
    )

    def save(self):
        from_address = 'adagios@adagios.opensource.is'
        to_address = ["palli@ok.is"]
        subject = _("Suggestion from Adagios")

        sender = self.cleaned_data['sender']
        topic = self.cleaned_data['topic']
        message = self.cleaned_data['message']

        msg = _("""
        topic: %(topic)s
        from: %(sender)s

        %(message)s
        """) % {'topic': topic, 'sender': sender, 'message': message}
        send_mail(subject, msg, from_address, to_address, fail_silently=False)


class UserdataForm(forms.Form):
    language = forms.ChoiceField(
        choices=settings.LANGUAGES,
        required=False
    )
    theme = forms.ChoiceField(
        choices=[(x, x) for x in adagios.utils.get_available_themes()],
        required=False
    )
    refresh_rate = forms.IntegerField(
        help_text="For pages that auto-reload. Set the number of seconds to wait between page refreshes. "
                  "Set refresh rate to 0 to disable automatic refreshing.",
        required=False,
    )


class AdagiosSettingsForm(forms.Form):
    nagios_config = forms.CharField(
        required=False, initial=settings.nagios_config,
        help_text=_("Path to nagios configuration file. i.e. /etc/nagios/nagios.cfg"))
    destination_directory = forms.CharField(
        required=False, initial=settings.destination_directory, help_text=_("Where to save new objects that adagios creates."))
    nagios_url = forms.CharField(required=False, initial=settings.nagios_url,
                                 help_text=_("URL (relative or absolute) to your nagios webcgi. Adagios will use this to make it simple to navigate from a configured host/service directly to the cgi."))
    nagios_service = forms.CharField(
        required=False,
        help_text=_("The name of the nagios service, commonly nagios or nagios3. Adagios will use this when stopping/starting/reloading nagios"))
    nagios_init_script = forms.CharField(
        required=False,
        help_text=_("You should define either 'Nagios service' or 'Nagios init script'. Path to you nagios init script. Adagios will use this when stopping/starting/reloading nagios"))
    nagios_binary = forms.CharField(
        help_text=_("Path to you nagios daemon binary. Adagios will use this to verify config with 'nagios -v nagios_config'"))
    livestatus_path = forms.CharField(
        help_text=_("Path to MK Livestatus socket. If left empty Adagios will try to autodiscover from your nagios.cfg"),
        required=False,
    )
    livestatus_limit = forms.IntegerField(
        help_text=_("Limit the number of rows shown per page in the status view to this."),
    )
    enable_githandler = forms.BooleanField(
        required=False, initial=settings.enable_githandler, help_text=_("If set. Adagios will commit any changes it makes to git repository."))
    enable_loghandler = forms.BooleanField(
        required=False, initial=settings.enable_loghandler, help_text=_("If set. Adagios will log any changes it makes to a file."))
    enable_authorization = forms.BooleanField(
        required=False, initial=settings.enable_authorization,
        help_text=_("If set. Users in Status view will only see hosts/services they are a contact for. Unset means everyone will see everything."))
    enable_status_view = forms.BooleanField(
        required=False, initial=settings.enable_status_view,
        help_text=_("If set. Enable status view which is an alternative to nagios legacy web interface. You will need to restart web server for the changes to take effect"))
    enable_local_logs = forms.BooleanField(
        required=False, initial=settings.enable_local_logs,
        help_text=_("If set, allow adagios to read logfiles from the monitoring engine. Might be performance sensitive)"))
    auto_reload = forms.BooleanField(
        required=False, initial=settings.auto_reload,
        help_text=_("If set. Nagios is reloaded automatically after every change."))
    warn_if_selinux_is_active = forms.BooleanField(
        required=False, help_text=_("Adagios does not play well with SElinux. So lets issue a warning if it is active. Only disable this if you know what you are doing."))
    pnp_filepath = forms.CharField(
        help_text=_("Full path to your pnp4nagios/index.php file. Adagios will use this to generate graphs"))
    pnp_url = forms.CharField(
        help_text=_("Full or relative url to pnp4nagios web interface, adagios can use this to link directly to pnp"))
    map_center = forms.CharField(
        help_text=_("Default coordinates when opening up the world map. This should be in the form of longitude,latitude"))
    map_zoom = forms.CharField(
        help_text=_("Default Zoom level when opening up the world map. 10 is a good default value"))
    language = forms.ChoiceField(choices=settings.LANGUAGES, required=False)
    theme = forms.ChoiceField(required=False, choices=[(x,x) for x in adagios.utils.get_available_themes()])
    refresh_rate = forms.IntegerField(
        help_text="For pages that auto-reload. Set the number of seconds to wait between page refreshes. "
                  "Set refresh rate to 0 to disable automatic refreshing."
    )
    enable_pnp4nagios = forms.BooleanField(required=False, help_text="If set, include pnp4nagios graphs in status views")
    enable_graphite = forms.BooleanField(required=False, help_text="If set, include graphite graphs in status views")
    graphite_url = forms.CharField(help_text="Path to your graphite install.", required=False)
    graphite_querystring = forms.CharField(help_text="Querystring that is passed into graphite's /render method. {host} is replaced with respective hostname while {host_} will apply common graphite escaping. i.e. example.com -> example_com", required=False)
    graphite_title = forms.CharField(help_text="Use this title on all graphs coming from graphite", required=False)
    default_host_template = forms.CharField(help_text="Use this template by default when adding new hosts.")
    default_service_template = forms.CharField(help_text="Use this template by default when adding new services.")
    default_contact_template = forms.CharField(help_text="Use this template by default when adding new contacts.")
    include = forms.CharField(
        required=False, help_text=_("Include configuration options from files matching this pattern"))

    def save(self):
        # First of all, if configfile does not exist, lets try to create it:
        if not os.path.isfile(settings.adagios_configfile):
            open(settings.adagios_configfile, 'w').write(
                _("# Autocreated by adagios"))
        for k, v in self.cleaned_data.items():
            Model.config._edit_static_file(
                attribute=k, new_value=v, filename=settings.adagios_configfile)
            self.adagios_configfile = settings.adagios_configfile
            #settings.__dict__[k] = v

    def __init__(self, *args, **kwargs):
        # Since this form is always bound, lets fetch current configfiles and
        # prepare them as post:
        if 'data' not in kwargs or kwargs['data'] == '':
            kwargs['data'] = settings.__dict__
        super(self.__class__, self).__init__(*args, **kwargs)

    def clean_pnp_filepath(self):
        filename = self.cleaned_data['pnp_filepath']
        return self.check_file_exists(filename)

    def clean_destination_directory(self):
        filename = self.cleaned_data['destination_directory']
        return self.check_file_exists(filename)

    def clean_nagios_init_script(self):
        filename = self.cleaned_data['nagios_init_script']
        if filename.startswith('sudo'):
            self.check_file_exists(filename.split()[1])
        elif filename:
            self.check_file_exists(filename)
        return filename

    def clean_nagios_binary(self):
        filename = self.cleaned_data['nagios_binary']
        return self.check_file_exists(filename)

    def clean_nagios_config(self):
        filename = self.cleaned_data['nagios_config']
        return self.check_file_exists(filename)

    def check_file_exists(self, filename):
        """ Raises validation error if filename does not exist """
        if not os.path.exists(filename):
            raise forms.ValidationError('No such file or directory')
        return filename

    def clean(self):
        cleaned_data = super(self.__class__, self).clean()
        for k, v in cleaned_data.items():
            # Convert all unicode to quoted strings
            if type(v) == type(u''):
                cleaned_data[k] = str('''"%s"''' % v)
            # Convert all booleans to True/False strings
            elif type(v) == type(False):
                cleaned_data[k] = str(v)
        return cleaned_data


class EditAllForm(forms.Form):

    """ This form intelligently modifies all attributes of a specific type.


    """

    def __init__(self, object_type, attribute, new_value, *args, **kwargs):
        self.object_type = object_type
        self.attribute = attribute
        self.new_value = new_value
        super(self.__class__, self).__init__(self, args, kwargs)
        search_filter = {}
        search_filter['object_type'] = object_type
        search_filter['%s__isnot' % attribute] = new_value
        items = Model.ObjectDefinition.objects.filter(**search_filter)
        interesting_objects = []
        for i in items:
            if attribute in i._defined_attributes or i.use is None:
                interesting_objects.append(i)
        self.interesting_objects = interesting_objects
        for i in interesting_objects:
            self.fields['modify_%s' % i.get_id()] = forms.BooleanField(
                required=False, initial=True)


class PNPActionUrlForm(forms.Form):

    """ This form handles applying action_url to bunch of hosts and services """
    #apply_action_url = forms.BooleanField(required=False,initial=True,help_text="If set, apply action_url to every service object in nagios")
    action_url = forms.CharField(
        required=False, initial="/pnp4nagios/graph?host=$HOSTNAME$&srv=$SERVICEDESC$",
        help_text=_("Reset the action_url attribute of every service check in your nagios configuration with this one. "))

    def save(self):
        action_url = self.cleaned_data['action_url']
        services = Model.Service.objects.filter(action_url__isnot=action_url)
        self.total_services = len(services)
        self.error_services = 0
        for i in services:
            if 'action_url' in i._defined_attributes or i.use is None:
                i.action_url = action_url
                try:
                    i.save()
                except Exception:
                    self.error_services += 1


class PNPTemplatesForm(forms.Form):

    """ This form manages your pnp4nagios templates """

    def __init__(self, *args, **kwargs):
        self.template_directories = []
        self.templates = []
        tmp = Model.config._load_static_file('/etc/pnp4nagios/config.php')
        for k, v in tmp:
            if k == "$conf['template_dirs'][]":
                # strip all ' and " from directory
                directory = v.strip(";").strip('"').strip("'")
                self.template_directories.append(directory)
                if os.path.isdir(directory):
                    for f in os.listdir(directory):
                        self.templates.append("%s/%s" % (directory, f))

        super(self.__class__, self).__init__(*args, **kwargs)


class PNPConfigForm(forms.Form):

    """ This form handles the npcd.cfg configuration file """
    user = forms.CharField(
        help_text=_("npcd service will have privileges of this group"))
    group = forms.CharField(
        help_text=_("npcd service will have privileges of this user"))
    log_type = forms.ChoiceField(
        widget=forms.RadioSelect, choices=pnp_log_type_choices, help_text=_("Define if you want to log to 'syslog' or 'file'"))
    log_file = forms.CharField(
        help_text=_("If log_type is set to file. Log to this file"))
    max_logfile_size = forms.IntegerField(
        help_text=_("Defines the maximum filesize (bytes) before logfile will rotate."))
    log_level = forms.ChoiceField(
        help_text=_("How much should we log?"), choices=pnp_loglevel_choices)
    perfdata_spool_dir = forms.CharField(
        help_text=_("where we can find the performance data files"))
    perfdata_file_run_cmd = forms.CharField(
        help_text=_("execute following command for each found file in perfdata_spool_dir"))
    perfdata_file_run_cmd_args = forms.CharField(
        required=False, help_text=_("optional arguments to perfdata_file_run_cmd"))
    identify_npcd = forms.ChoiceField(widget=forms.RadioSelect, choices=(
        ('1', 'Yes'), ('0', 'No')), help_text=_("If yes, npcd will append -n to the perfdata_file_run_cmd"))
    npcd_max_threads = forms.IntegerField(
        help_text=_("Define how many parallel threads we should start"))
    sleep_time = forms.IntegerField(
        help_text=_("How many seconds npcd should wait between dirscans"))
    load_threshold = forms.FloatField(
        help_text=_("npcd won't start if load is above this threshold"))
    pid_file = forms.CharField(help_text=_("Location of your pid file"))
    perfdata_file = forms.CharField(
        help_text=_("Where should npcdmod.o write the performance data. Must not be same directory as perfdata_spool_dir"))
    perfdata_spool_filename = forms.CharField(
        help_text=_("Filename for the spooled files"))
    perfdata_file_processing_interval = forms.IntegerField(
        help_text=_("Interval between file processing"))

    def __init__(self, initial=None, *args, **kwargs):
        if not initial:
            initial = {}
        my_initial = {}
        # Lets use PNPBrokerModuleForm to find sensible path to npcd config
        # file
        broker_form = PNPBrokerModuleForm()
        self.npcd_cfg = broker_form.initial.get('config_file')
        npcd_values = Model.config._load_static_file(self.npcd_cfg)
        for k, v in npcd_values:
            my_initial[k] = v
        super(self.__class__, self).__init__(
            initial=my_initial, *args, **kwargs)

    def save(self):
        for i in self.changed_data:
            Model.config._edit_static_file(
                attribute=i, new_value=self.cleaned_data[i], filename=self.npcd_cfg)


class EditFileForm(forms.Form):

    """ Manages editing of a single file """
    filecontent = forms.CharField(widget=forms.Textarea(
        attrs={'wrap': 'off', 'rows': '50', 'cols': '2000'}))

    def __init__(self, filename, initial=None, *args, **kwargs):
        if not initial:
            initial = {}
        self.filename = filename
        my_initial = initial.copy()
        if 'filecontent' not in my_initial:
            my_initial['filecontent'] = open(filename).read()
        super(self.__class__, self).__init__(
            initial=my_initial, *args, **kwargs)

    def save(self):
        if 'filecontent' in self.changed_data:
            data = self.cleaned_data['filecontent']
            open(self.filename, 'w').write(data)


class PNPBrokerModuleForm(forms.Form):

    """ This form is responsible for configuring PNP4Nagios. """
    #enable_pnp= forms.BooleanField(required=False, initial=True,help_text="If set, PNP will be enabled and will graph Nagios Performance Data.")
    broker_module = forms.CharField(
        help_text=_("Full path to your npcdmod.o broker module that shipped with your pnp4nagios installation"))
    config_file = forms.CharField(
        help_text=_("Full path to your npcd.cfg that shipped with your pnp4nagios installation"))
    event_broker_options = forms.IntegerField(
        initial="-1", help_text=_("Nagios's default of -1 is recommended here. PNP Documentation says you will need at least bits 2 and 3. Only change this if you know what you are doing."))
    process_performance_data = forms.BooleanField(
        required=False, initial=True, help_text=_("PNP Needs the nagios option process_performance_data enabled to function. Make sure it is enabled."))
    #apply_action_url = forms.BooleanField(required=False,initial=True,help_text="If set, apply action_url to every service object in nagios")
    #action_url=forms.CharField(required=False,initial="/pnp4nagios/graph?host=$HOSTNAME$&srv=$SERVICEDESC$", help_text="Action url that your nagios objects can use to access perfdata")

    def clean_broker_module(self):
        """ Raises validation error if filename does not exist """
        filename = self.cleaned_data['broker_module']
        if not os.path.exists(filename):
            raise forms.ValidationError('File not found')
        return filename

    def clean_config_file(self):
        """ Raises validation error if filename does not exist """
        filename = self.cleaned_data['config_file']
        if not os.path.exists(filename):
            raise forms.ValidationError('File not found')
        return filename

    def __init__(self, initial=None, *args, **kwargs):
        if not initial:
            initial = {}
        my_initial = {}
        Model.config.parse()
        maincfg_values = Model.config.maincfg_values
        self.nagios_configline = None
        for k, v in Model.config.maincfg_values:
            if k == 'broker_module' and v.find('npcdmod.o') > 0:
                self.nagios_configline = v
                v = v.split()
                my_initial['broker_module'] = v.pop(0)
                for i in v:
                    if i.find('config_file=') > -1:
                        my_initial['config_file'] = i.split('=', 1)[1]
            elif k == "event_broker_options":
                my_initial[k] = v
        # If view specified any initial values, they overwrite ours
        for k, v in initial.items():
            my_initial[k] = v
        if 'broker_module' not in my_initial:
            my_initial['broker_module'] = self.get_suggested_npcdmod_path()
        if 'config_file' not in my_initial:
            my_initial['config_file'] = self.get_suggested_npcd_path()
        super(self.__class__, self).__init__(
            initial=my_initial, *args, **kwargs)

    def get_suggested_npcdmod_path(self):
        """ Returns best guess for full path to npcdmod.o file """
        possible_locations = [
            "/usr/lib/pnp4nagios/npcdmod.o",
            "/usr/lib64/nagios/brokers/npcdmod.o",
        ]
        for i in possible_locations:
            if os.path.isfile(i):
                return i
        return possible_locations[-1]

    def get_suggested_npcd_path(self):
        """ Returns best guess for full path to npcd.cfg file """
        possible_locations = [
            "/etc/pnp4nagios/npcd.cfg"
        ]
        for i in possible_locations:
            if os.path.isfile(i):
                return i
        return possible_locations[-1]

    def save(self):
        if 'broker_module' in self.changed_data or 'config_file' in self.changed_data or self.nagios_configline is None:
            v = "%s config_file=%s" % (
                self.cleaned_data['broker_module'], self.cleaned_data['config_file'])
            Model.config._edit_static_file(
                attribute="broker_module", new_value=v, old_value=self.nagios_configline, append=True)

        # We are supposed to handle process_performance_data attribute.. lets
        # do that here
        process_performance_data = "1" if self.cleaned_data[
            'process_performance_data'] else "0"
        Model.config._edit_static_file(
            attribute="process_performance_data", new_value=process_performance_data)

        # Update event broker only if it has changed
        name = "event_broker_options"
        if name in self.changed_data:
            Model.config._edit_static_file(
                attribute=name, new_value=self.cleaned_data[name])


class PluginOutputForm(forms.Form):
    plugin_output = forms.CharField(
        widget=forms.Textarea(attrs={'wrap': 'off', 'cols': '80'}))

    def parse(self):
        from pynag import Utils
        plugin_output = self.cleaned_data['plugin_output']
        output = Utils.PluginOutput(plugin_output)
        self.results = output


class NagiosServiceForm(forms.Form):

    """ Maintains control of the nagios service / reload / restart / etc """
    #path_to_init_script = forms.CharField(help_text="Path to your nagios init script", initial=NAGIOS_INIT)
    #nagios_binary = forms.CharField(help_text="Path to your nagios binary", initial=NAGIOS_BIN)
    #command = forms.ChoiceField(choices=COMMAND_CHOICES)

    def save(self):
        #nagios_bin = self.cleaned_data['nagios_bin']
        daemon = adagios.daemon.Daemon()
        if "reload" in self.data:
            command = daemon.reload
            self.command = "reload"
        elif "restart" in self.data:
            command = daemon.restart
            self.command = "restart"
        elif "stop" in self.data:
            command = daemon.stop
            self.command = "stop"
        elif "start" in self.data:
            command = daemon.start
            self.command = "start"
        elif "status" in self.data:
            command = daemon.status
            self.command = "status"
        elif "verify" in self.data:
            command = daemon.verify_config
            self.command = "verify"
        else:
            raise Exception(_("Unknown command"))

        code = command()
        self.stdout = daemon.stdout or ""
        self.stderr = daemon.stderr or ""
        self.exit_code = code

    def verify(self):
        """ Run "nagios -v nagios.cfg" and returns errors/warning

        Returns:
        [
            {'errors': []},
            {'warnings': []}
        ]
        """
        nagios_binary = settings.nagios_binary
        nagios_config = settings.nagios_config
        command = "%s -v '%s'" % (nagios_binary, nagios_config)
        code, stdout, stderr = pynag.Utils.runCommand(command)
        self.stdout = stdout or None
        self.stderr = stderr or None
        self.exit_code = code

        for line in stdout.splitlines():
            line = line.strip()
            warnings = []
            errors = []
            if line.lower.startswith('warning:'):
                warning = {}


class SendEmailForm(forms.Form):

    """ Form used to send email to one or more contacts regarding particular services
    """
    to = forms.CharField(
        required=True,
        help_text=_("E-mail address"),
    )
    message = forms.CharField(
        widget=forms.widgets.Textarea(attrs={'rows': 15, 'cols': 40}),
        required=False,
        help_text=_("Message that is to be sent to recipients"),
    )
    add_myself_to_cc = forms.BooleanField(
        required=False,
        help_text=_("If checked, you will be added automatically to CC")
    )
    acknowledge_all_problems = forms.BooleanField(
        required=False,
        help_text=_("If checked, also acknowledge all problems as they are sent")
    )

    def __init__(self, remote_user, *args, **kwargs):
        """ Create a new instance of SendEmailForm, contact name and email is used as from address.
        """
        self.remote_user = remote_user
        #self.contact_email = contact_email
        self.html_content = _("There is now HTML content with this message.")
        self.services = []
        self.hosts = []
        self.status_objects = []
        self._resolve_remote_user(self.remote_user)
        super(self.__class__, self).__init__(*args, **kwargs)

    def save(self):

        subject = _("%s sent you a a message through adagios") % self.remote_user

        cc_address = []
        from_address = self._resolve_remote_user(self.remote_user)
        # Check if _resolve_remote_user did in fact return an email address - avoid SMTPSenderRefused.
        import re # re built in Py1.5+
        if re.compile('([\w\-\.]+@(\w[\w\-]+\.)+[\w\-]+)').search(from_address) is None:
            from_address = str(from_address) + '@no.domain'
        to_address = self.cleaned_data['to']
        to_address = to_address.split(',')
        text_content = self.cleaned_data['message']
        text_content = text_content.replace('\n','<br>')

        # self.html_content is rendered in misc.views.mail()
        html_content = smart_str(text_content) + "<p></p>" + smart_str(self.html_content)
        if self.cleaned_data['add_myself_to_cc']:
            cc_address.append(from_address)
        if self.cleaned_data['acknowledge_all_problems']:
            comment = _("Sent mail to %s") % self.cleaned_data['to']
            self.acknowledge_all_services(comment)
            self.acknowledge_all_hosts(comment)
        # Here we actually send some email:

        msg = EmailMultiAlternatives(
            subject=subject, body=text_content, from_email=from_address, cc=cc_address, to=to_address)
        msg.attach_alternative(html_content, "text/html")
        msg.send()

    def acknowledge_all_hosts(self, comment):
        """ Acknowledge all problems in self.hosts
        """
        for i in self.hosts:
            host_name = i.get('host_name')
            sticky = "1"
            persistent = "0"
            notify = "0"
            author = self.remote_user

            pynag.Control.Command.acknowledge_host_problem(host_name=host_name,
                                                          sticky=sticky,
                                                          persistent=persistent,
                                                          notify=notify,
                                                          author=author,
                                                          comment=comment)
    def acknowledge_all_services(self, comment):
        """ Acknowledge all problems in self.services
        """
        for i in self.services:
            host_name = i.get('host_name')
            service_description = i.get('description')
            sticky = "1"
            persistent = "0"
            notify = "0"
            author = self.remote_user

            pynag.Control.Command.acknowledge_svc_problem(host_name=host_name,
                                                          service_description=service_description,
                                                          sticky=sticky,
                                                          persistent=persistent,
                                                          notify=notify,
                                                          author=author,
                                                          comment=comment)

    def _resolve_remote_user(self, username):
        """ Returns a valid "Full Name <email@example.com>" for remote http authenticated user.
         If Remote user is a nagios contact, then return: Contact_Alias <contact_email>"
         Else if remote user is a valid email address, return that address
         Else return None
        """
        import adagios.status.utils
        livestatus = adagios.status.utils.livestatus(request=None)
        try:
            contact = livestatus.get_contact(username)
            return "%s <%s>" % (contact.get('alias'), contact.get('email'))
        except IndexError:
            # If we get here, then remote_user does not exist as a contact.
            return username





class PasteForm(forms.Form):
    paste = forms.CharField(initial=initial_paste, widget=forms.Textarea())

    def parse(self):
        c = pynag.Parsers.config()
        self.config = c
        c.reset()
        paste = self.cleaned_data['paste']
        # Also convert raw paste into a string so we can display errors at the
        # right place:
        self.pasted_string = paste.splitlines()
        items = c.parse_string(paste)
        c.pre_object_list = items
        c._post_parse()
        all_objects = []
        for object_type, objects in c.data.items():
            model = pynag.Model.string_to_class.get(
                object_type, pynag.Model.ObjectDefinition)
            for i in objects:
                Class = pynag.Model.string_to_class.get(
                    i['meta']['object_type'])
                my_object = Class(item=i)
                all_objects.append(my_object)
        self.objects = all_objects



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.context_processors import csrf
from django.shortcuts import render_to_response
from django.shortcuts import HttpResponse
import adagios.settings
import adagios.status.utils
import os

from adagios.views import adagios_decorator, error_page
from django.template import RequestContext
from adagios.contrib import get_template_name
from django import template
from django.utils.translation import ugettext as _


@adagios_decorator
def index(request, contrib_dir=None):
    """ List all available user contributed views in adagios.settings.contrib_dir """
    messages = []
    errors = []

    if not contrib_dir:
        contrib_dir = adagios.settings.contrib_dir
    views = os.listdir(contrib_dir)

    if not views:
        errors.append(_("Directory '%s' is empty") % contrib_dir)
    return render_to_response("contrib_index.html", locals(), context_instance=RequestContext(request))


@adagios_decorator
def contrib(request, arg1, arg2=None, arg3=None, arg4=None):
    messages = []
    errors = []

    full_path = get_template_name(adagios.settings.contrib_dir, arg1, arg2, arg3, arg4)
    if os.path.isdir(full_path):
        return index(request, contrib_dir=full_path)

    with open(full_path) as f:
        content = f.read()

    # Lets populate local namespace with convenient data
    services = lambda: locals().get('services', adagios.status.utils.get_services(request))
    hosts = lambda: locals().get('hosts', adagios.status.utils.get_hosts(request))
    service_problems = lambda: locals().get('service_problems', adagios.status.utils.get_hosts(request, state__isnot='0'))
    host_problems = lambda: locals().get('host_problems', adagios.status.utils.get_hosts(request, state__isnot='0'))
    statistics = lambda: locals().get('statistics', adagios.status.utils.get_statistics(request))

    t = template.Template(content)
    c = RequestContext(request, locals())
    html = t.render(c)
    return HttpResponse(html)



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2012, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

This is a rest interface used by the "/rest/" module that affects adagios directly.

"""

from adagios import __version__, notifications, tasks
from adagios.settings import plugins
from adagios import userdata
from django.utils.translation import ugettext as _

version = __version__


def add_notification(level="info", message="message", notification_id=None, notification_type=None, user=None):
    """ Add a new notification to adagios notification bar.

    Arguments:
      level                      -- pick "info" "success" "error" "danger"
      message                    -- Arbitary text message,
      notification_id (optional) -- Use this if you want to remote
                                 -- remove this notification later via clear_notification()
      notification_type          -- Valid options: "generic" and "show_once"

      user                       -- If specified, only display notification for this specific user.

    Returns:
      None

    Examples:
    >>> add_notification(level="warning", message="Nagios needs to reload")
    """
    if not notification_id:
        notification_id = str(message.__hash__())
    if not notification_type:
        notification_type = "generic"
    notification = locals()
    notifications[notification_id] = notification


def clear_notification(notification_id):
    """ Clear one notification from adagios notification panel """
    if notification_id in notifications:
        del notifications[notification_id]
        return "success"
    return "not found"


def get_notifications(request):
    """ Shows all current notifications """
    result = []
    for k in notifications.keys():
        i = notifications[k]
        if i.get('user') and i.get('user') != request.META.get('remote_user'):
            continue # Skipt this message if it is meant for someone else
        elif i.get('notification_type') == 'show_once':
            del notifications[k]
            pass
        result.append(i)
    return result


def clear_all_notifications():
    """ Removes all notifications from adagios notification panel """
    notifications.clear()
    return "all notifications cleared"


def list_tasks():
    """

    """
    result = []
    for task in tasks:
        current_task = {
            'task_id': task.get_id(),
            'task_status': task.status()
            }
        result.append(current_task)
    return result


def get_task(task_id="someid"):
    """ Return information about one specific background task """
    for task in tasks:
        if str(task.get_id) == str(task_id) or task_id:
            current_task = {
                'task_id': task.get_id(),
                'task_status': task.status()
            }
            return current_task
    raise KeyError(_("Task not '%s' Found") % task_id)


def get_user_preferences(request):
    try:
        user = userdata.User(request)
    except Exception as e:
        raise e
    return user.to_dict()


def set_user_preference(request, **kwargs):
    try:
        user = userdata.User(request)
    except Exception as e:
        raise e
    
    for (k, v) in kwargs.iteritems():
        if not k.startswith('_'):
            user.set_pref(k, v)
    user.save()


def save_search(request, name, url):
    """Create a new saved search, which will be accessible from main menu."""
    user = userdata.User(request)
    user_data = user.to_dict()
    if not name:
        raise ValueError("You must provide a name for this search")
    if not url:
        raise ValueError("You must provide a url for this search")
    url = url.strip('#')
    saved_searches = user_data.get(userdata.User.SAVED_SEARCHES, {})
    saved_searches[name] = url
    user.set_pref(userdata.User.SAVED_SEARCHES, saved_searches)
    user.save()


def get_saved_searches(request):
    """Get a list of all saved searches for this user."""
    user_data = get_user_preferences(request)
    saved_searches = user_data.get(userdata.User.SAVED_SEARCHES, {})
    return saved_searches


def delete_saved_search(request, name):
    """Delete one specific saved search for this user."""
    user = userdata.User(request)
    user_data = user.to_dict()
    saved_searches = user_data.get(userdata.User.SAVED_SEARCHES, {})
    saved_searches.pop(name, None)
    user.set_pref(userdata.User.SAVED_SEARCHES, saved_searches)
    user.save()




# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2010, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.utils.safestring import mark_safe
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _

from pynag import Model
from pynag.Utils import AttributeList
from pynag.Utils import importer
from adagios.objectbrowser.help_text import object_definitions
from adagios.forms import AdagiosForm
import adagios.misc.rest
import adagios.settings

# These fields are special, they are a comma seperated list, and may or
# may not have +/- in front of them.
MULTICHOICE_FIELDS = ('servicegroups', 'hostgroups', 'contacts',
                      'contact_groups', 'contactgroups', 'use', 'notification_options')

SERVICE_NOTIFICATION_OPTIONS = (
    ('w', 'warning'),
    ('c', 'critical'),
    ('r', 'recovery'),
    ('u', 'unreachable'),
    ('d', 'downtime'),
    ('f', 'flapping'),
)

HOST_NOTIFICATION_OPTIONS = (
    ('d', 'down'),
    ('u', 'unreachable'),
    ('r', 'recovery'),
    ('f', 'flapping'),
    ('s', 'scheduled_downtime')
)

# All objects definition types in our config, in a form that is good for pynag forms
ALL_OBJECT_TYPES = sorted(map(lambda x: (x, x), Model.string_to_class.keys()))

# When boolean fields are set in configuration, generally 1=True and 0=False
BOOLEAN_CHOICES = (('', 'not set'), ('1', '1'), ('0', '0'))


_INITIAL_OBJECTS_IMPORT = """\
host_name,address,use

localhost,127.0.0.1,generic-host
otherhost,127.0.0.2,generic-host
"""


class PynagAutoCompleteField(forms.CharField):
    """ Behaves like Charfield, but includes data-choices for select2 autocomplete. """
    def __init__(self, object_type, inline_help_text=None, complete="shortname", *args, **kwargs):
        super(PynagAutoCompleteField, self).__init__(*args, **kwargs)

        # Add help text into data-placeholder
        if not inline_help_text:
            inline_help_text = "Select some {object_type}s"
            inline_help_text = inline_help_text.format(object_type=object_type)
        self.widget.attrs['data-placeholder'] = inline_help_text

        # Add autcomplete choices in data-choices
        if complete == 'shortname':
            choices = self.get_all_shortnames(object_type=object_type)
        elif complete == 'name':
            choices = self.get_all_object_names(object_type=object_type)
        else:
            raise ValueError("complete must be either shortname or name")
        choices_string = ','.join(choices)
        self.widget.attrs['data-choices'] = choices_string

        # Give our widget a unique css class
        self.widget.attrs['class'] = self.widget.attrs.get('class', '')
        self.widget.attrs['class'] += ' pynag-autocomplete '

        # Hardcode widget length to 500px, because select2 plays badly
        # with css
        self.widget.attrs['style'] = self.widget.attrs.get('style', '')
        self.widget.attrs['style'] += ' width: 500px; '

    def get_all_shortnames(self, object_type):
        """ Returns a list of all shortnames, given a specific object type."""
        objects = self.get_all_objects(object_type)
        shortnames = map(lambda x: x.get_shortname(), objects)

        # Remove objects with no shortname
        shortnames = filter(lambda x: x, shortnames)

        return shortnames

    def get_all_object_names(self, object_type):
        """ Returns a list of all object names (name attribute) for a given object type. """
        objects = self.get_all_objects(object_type)
        names = map(lambda x: x.name, objects)

        # Remove objects with no name
        names = filter(lambda x: x, names)
        return names

    def get_all_objects(self, object_type):
        """ Returns all object of a given object type """
        Class = Model.string_to_class[object_type]
        objects = Class.objects.get_all()
        return objects

    def prepare_value(self, value):
        """
        Takes a comma separated string, removes + if it is prefixed so. Returns a comma seperated string
        """
        if value == 'null':
            return value
        elif isinstance(value, basestring):
            a = AttributeList(value)
            self.__prefix = a.operator
            a.operator = ''
            a = str(a)
            value = a
        return value


class PynagChoiceField(forms.MultipleChoiceField):

    """ multichoicefields that accepts comma seperated input as values """

    def __init__(self, inline_help_text=_("Select some options"), *args, **kwargs):
        self.__prefix = ''
        self.data = kwargs.get('data')
        super(PynagChoiceField, self).__init__(*args, **kwargs)
        self.widget.attrs['data-placeholder'] = inline_help_text

    def clean(self, value):
        """
        Changes list into a comma separated string. Removes duplicates.
        """
        if not value:
            return "null"
        tmp = []
        for i in value:
            if i not in tmp:
                tmp.append(i)
        value = self.__prefix + ','.join(tmp)
        return value

    def prepare_value(self, value):
        """
        Takes a comma separated string, removes + if it is prefixed so. Returns a list
        """
        if value is None:
            return []
        if isinstance(value, str):
            self.attributelist = AttributeList(value)
            self.__prefix = self.attributelist.operator
            return self.attributelist.fields
        else:
            raise ValueError("Expected string. Got %s" % type(value))

    def set_prefix(self, value):
        self.__prefix = value

    def get_prefix(self):
        return self.__prefix


class PynagRadioWidget(forms.widgets.HiddenInput):

    """ Special Widget designed to make Nagios attributes with 0/1 values look like on/off buttons """

    def render(self, name, value, attrs=None):
        output = super(PynagRadioWidget, self).render(name, value, attrs)
        one, zero, unset = "", "", ""
        if value == "1":
            one = "active"
        elif value == "0":
            zero = "active"
        else:
            unset = "active"
        prefix = """
        <div class="btn-group" data-toggle-name="%s" data-toggle="buttons-radio">
          <button type="button" value="1" class="btn btn %s">On</button>
          <button type="button" value="0" class="btn btn %s">Off</button>
          <button type="button" value="" class="btn %s">Not set</button>
        </div>
        """ % (name, one, zero, unset)
        output += prefix
        return mark_safe(output)


class PynagForm(AdagiosForm):

    def clean(self):
        cleaned_data = super(PynagForm, self).clean()
        for k, v in cleaned_data.items():
            # change from unicode to str
            v = cleaned_data[k] = smart_str(v)

            # Empty string, or the string None, means remove the field
            if v in ('', 'None'):
                cleaned_data[k] = v = None

            # Maintain operator (+,-, !) for multichoice fields
            if k in MULTICHOICE_FIELDS and v and v != "null":
                operator = AttributeList(self.pynag_object.get(k, '')).operator or ''
                cleaned_data[k] = "%s%s" % (operator, v)
        return cleaned_data

    @property
    def changed_data(self):
        # Fields that do not appear in our POST are not marked as changed data
        # This can happen for example because some views decide not to print all
        # Fields to the browser
        changed_data = super(PynagForm, self).changed_data
        changed_data = filter(lambda x: x in self.data, changed_data)
        return changed_data

    def save(self):
        changed_keys = map(lambda x: smart_str(x), self.changed_data)
        for k in changed_keys:

            # Ignore fields that did not appear in the POST at all EXCEPT
            # If it it a pynagchoicefield. That is because multichoicefield that
            # does not appear in the post, means that the user removed every attribute
            # in the multichoice field
            if k not in self.data and not isinstance(self.fields.get(k, None), PynagChoiceField):
                continue

            value = self.cleaned_data[k]

            # Sometimes attributes slide in changed_data without having
            # been modified, lets ignore those
            if self.pynag_object[k] == value:
                continue

            # Multichoice fields have a special restriction, sometimes they contain
            # the same values as before but in a different order.
            if k in MULTICHOICE_FIELDS:
                original = AttributeList(self.pynag_object[k])
                new = AttributeList(value)
                if sorted(original.fields) == sorted(new.fields):
                    continue            # If we reach here, it is save to modify our pynag object.

            # Here we actually make a change to our pynag object
            self.pynag_object[k] = value
            # Additionally, update the field for the return form
            self.fields[k] = self.get_pynagField(k, css_tag="defined")
            self.fields[k].value = value

        try:
            self.pynag_object.save()
            adagios.misc.rest.add_notification(message=_(
                "Object successfully saved"),
                level="success",
                notification_type="show_once")
        except IOError:
            adagios.misc.rest.add_notification(message=_(
                "Object cannot be saved : Permission denied on file system"),
                level="danger",
                notification_type="show_once")

    def __init__(self, pynag_object, *args, **kwargs):
        self.pynag_object = p = pynag_object
        super(PynagForm, self).__init__(*args, **kwargs)
        # Lets find out what attributes to create
        object_type = pynag_object['object_type']
        defined_attributes = sorted(p._defined_attributes.keys())
        inherited_attributes = sorted(p._inherited_attributes.keys())
        all_attributes = sorted(object_definitions.get(object_type).keys())
        all_attributes += ['name', 'use', 'register']

        # Special hack for macros
        # If this is a post and any post data looks like a nagios macro
        # We will generate a field for it on the fly
        macros = filter(lambda x: x.startswith('$') and x.endswith('$'), self.data.keys())
        for field_name in macros:
            # if field_name.startswith('$ARG'):
            #    self.fields[field_name] = self.get_pynagField(field_name, css_tag='defined')
            if object_type == 'service' and field_name.startswith('$_SERVICE'):
                self.fields[field_name] = self.get_pynagField(field_name, css_tag='defined')
            elif object_type == 'host' and field_name.startswith('$_HOST'):
                self.fields[field_name] = self.get_pynagField(field_name, css_tag='defined')

        # Calculate what attributes are "undefined"
        self.undefined_attributes = []
        for i in all_attributes:
            if i in defined_attributes:
                continue
            if i in inherited_attributes:
                continue
            self.undefined_attributes.append(i)
        # Find out which attributes to show
        for field_name in defined_attributes:
            self.fields[field_name] = self.get_pynagField(field_name, css_tag='defined')
        for field_name in inherited_attributes:
            self.fields[field_name] = self.get_pynagField(field_name, css_tag="inherited")
        for field_name in self.undefined_attributes:
            self.fields[field_name] = self.get_pynagField(field_name, css_tag='undefined')

        # If no initial values were provided, use the one in our pynag object.
        if not self.initial:
            self.initial = pynag_object._defined_attributes
            self.initial.update(pynag_object._changes)

        for name, field in self.fields.items():
            if name in self.initial:
                field.initial = self.initial[name]
            elif name in self.pynag_object:
                field.initial = self.pynag_object[name]

    def get_pynagField(self, field_name, css_tag="", required=None):
        """ Takes a given field_name and returns a forms.Field that is appropriate for this field

          Arguments:
            field_name  --  Name of the field to add, example "host_name"
            css_tag     --  String will make its way as a css attribute in the resulting html
            required    --  If True, make field required. If None, let pynag decide
        """
        # Lets figure out what type of field this is, default to charfield
        object_type = self.pynag_object['object_type']
        definitions = object_definitions.get(object_type) or {}
        options = definitions.get(field_name) or {}

        # Find out what type of field to create from the field_name.
        # Lets assume charfield in the beginning
        field = forms.CharField()

        if False is True:
            pass
        elif field_name in ('contact_groups', 'contactgroups', 'contactgroup_members'):
            field = PynagAutoCompleteField(object_type='contactgroup', complete='shortname')
        elif field_name == 'use':
            field = PynagAutoCompleteField(object_type=object_type, complete="name")
        elif field_name in ('servicegroups', 'servicegroup_members'):
            field = PynagAutoCompleteField(object_type='servicegroup')
        elif field_name in ('hostgroups', 'hostgroup_members', 'hostgroup_name') and object_type != 'hostgroup':
            field = PynagAutoCompleteField(object_type='hostgroup')
        elif field_name == 'members' and object_type == 'hostgroup':
            field = PynagAutoCompleteField(object_type='host')
        elif field_name == 'host_name' and object_type == 'service':
            field = PynagAutoCompleteField(object_type='host')
        elif field_name in ('contacts', 'members'):
            field = PynagAutoCompleteField(object_type='contact')
        elif field_name.endswith('_period'):
            all_objects = Model.Timeperiod.objects.filter(
                timeperiod_name__contains='')
            choices = [('', '')] + map(
                lambda x: (x.timeperiod_name, x.timeperiod_name), all_objects)
            field = forms.ChoiceField(choices=sorted(choices))
        elif field_name.endswith('notification_commands'):
            all_objects = Model.Command.objects.filter(
                command_name__contains='')
            choices = [('', '')] + map(
                lambda x: (x.command_name, x.command_name), all_objects)
            field = PynagChoiceField(choices=sorted(choices))
        # elif field_name == 'check_command':
        #    all_objects = Model.Command.objects.all
        #    choices = [('','')] + map(lambda x: (x.command_name, x.command_name), all_objects)
        #    field = forms.ChoiceField(choices=sorted(choices))
        elif field_name.endswith('notification_options') and self.pynag_object.object_type == 'host':
            field = PynagChoiceField(
                choices=HOST_NOTIFICATION_OPTIONS, inline_help_text=_("No %(field_name)s selected") % {'field_name': field_name})
        elif field_name.endswith('notification_options') and self.pynag_object.object_type == 'service':
            field = PynagChoiceField(
                choices=SERVICE_NOTIFICATION_OPTIONS, inline_help_text=_("No %(field_name)s selected") % {'field_name': field_name})
        elif options.get('value') == '[0/1]':
            field = forms.CharField(widget=PynagRadioWidget)

        # Lets see if there is any help text available for our field
        if field_name in object_definitions[object_type]:
            help_text = object_definitions[object_type][field_name].get(
                'help_text', _("No help available for this item"))
            field.help_text = help_text

        # No prettyprint for macros
        if field_name.startswith('_'):
            field.label = field_name

        # If any CSS tag was given, add it to the widget
        self.add_css_tag(field=field, css_tag=css_tag)

        if 'required' in options:
            self.add_css_tag(field=field, css_tag=options['required'])
            field.required = options['required'] == 'required'
        else:
            field.required = False

        # At the moment, our database of required objects is incorrect
        # So if caller did not specify if field is required, we will not
        # make it required
        if required is None:
            field.required = False
        else:
            field.required = required

        # Put inherited value in the placeholder
        inherited_value = self.pynag_object._inherited_attributes.get(
            field_name)
        if inherited_value is not None:
            self.add_placeholder(
                field,
                _('%(inherited_value)s (inherited from template)') % {'inherited_value': smart_str(inherited_value)}
            )

        if field_name in MULTICHOICE_FIELDS:
            self.add_css_tag(field=field, css_tag="multichoice")

        return field

    def add_css_tag(self, field, css_tag):
        """ Add a CSS tag to the widget of a specific field """
        if not 'class' in field.widget.attrs:
            field.widget.attrs['class'] = ''
            field.css_tag = ''
        field.widget.attrs['class'] += " " + css_tag
        if not hasattr(field, 'css_tag'):
            field.css_tag = ''
        field.css_tag += " " + css_tag

    def add_placeholder(self, field, placeholder=_("Insert some value here")):
        field.widget.attrs['placeholder'] = placeholder
        field.placeholder = placeholder


class AdvancedEditForm(AdagiosForm):

    """ A form for pynag.Model.Objectdefinition

    This form will display a charfield for every attribute of the objectdefinition

    "Every" attribute means:
    * Every defined attribute
    * Every inherited attribute
    * Every attribute that is defined in nagios object definition html

    """
    register = forms.CharField(
        required=False, help_text=_("Set to 1 if you want this object enabled."))
    name = forms.CharField(required=False, label=_("Generic Name"),
                           help_text=_("This name is used if you want other objects to inherit (with the use attribute) what you have defined here."))
    use = forms.CharField(required=False, label=_("Use"),
                          help_text=_("Inherit all settings from another object"))
    __prefix = "advanced"  # This prefix will go on every field

    def save(self):
        for k in self.changed_data:
            # change from unicode to str
            value = smart_str(self.cleaned_data[k])
            # same as original, lets ignore that
            if self.pynag_object[k] == value:
                continue
            if value == '':
                value = None

            # If we reach here, it is save to modify our pynag object.
            self.pynag_object[k] = value
        self.pynag_object.save()

    def clean(self):
        cleaned_data = super(AdvancedEditForm, self).clean()
        for k, v in cleaned_data.items():
            # change from unicode to str
            cleaned_data[k] = smart_str(v)
        return cleaned_data

    def __init__(self, pynag_object, *args, **kwargs):
        self.pynag_object = pynag_object
        super(AdvancedEditForm, self).__init__(
            *args, prefix=self.__prefix, **kwargs)

        # Lets find out what attributes to create
        object_type = pynag_object['object_type']
        all_attributes = sorted(object_definitions.get(object_type).keys())
        for field_name in self.pynag_object.keys() + all_attributes:
            if field_name == 'meta':
                continue
            help_text = ""
            if field_name in object_definitions[object_type]:
                help_text = object_definitions[object_type][field_name].get(
                    'help_text', _("No help available for this item"))
            self.fields[field_name] = forms.CharField(
                required=False, label=field_name, help_text=help_text)
        self.fields.keyOrder = sorted(self.fields.keys())


class GeekEditObjectForm(AdagiosForm):
    definition = forms.CharField(
        widget=forms.Textarea(attrs={'wrap': 'off', 'cols': '80'}))

    def __init__(self, pynag_object=None, *args, **kwargs):
        self.pynag_object = pynag_object
        super(GeekEditObjectForm, self).__init__(*args, **kwargs)

    def clean_definition(self, value=None):
        definition = smart_str(self.cleaned_data['definition'])
        definition = definition.replace('\r\n', '\n')
        definition = definition.replace('\r', '\n')
        if not definition.endswith('\n'):
            definition += '\n'
        return definition

    def save(self):
        definition = self.cleaned_data['definition']
        self.pynag_object.rewrite(str_new_definition=definition)


class DeleteObjectForm(AdagiosForm):

    """ Form used to handle deletion of one single object """

    def __init__(self, pynag_object, *args, **kwargs):
        self.pynag_object = pynag_object
        super(DeleteObjectForm, self).__init__(*args, **kwargs)
        if self.pynag_object.object_type == 'host':
            recursive = forms.BooleanField(
                required=False, initial=True, label=_("Delete Services"),
                help_text=_("Check this box if you also want to delete all services of this host"))
            self.fields['recursive'] = recursive

    def delete(self):
        """ Deletes self.pynag_object. """
        recursive = False
        if 'recursive' in self.cleaned_data and self.cleaned_data['recursive'] is True:
            recursive = True
        self.pynag_object.delete(recursive)


class CopyObjectForm(AdagiosForm):

    """ Form to assist a user to copy a single object definition
    """

    def __init__(self, pynag_object, *args, **kwargs):
        self.pynag_object = pynag_object
        super(CopyObjectForm, self).__init__(*args, **kwargs)
        object_type = pynag_object['object_type']

        # For templates we assume the new copy will have its generic name changed
        # otherwise we display different field depending on what type of an
        # object it is
        if pynag_object['register'] == '0':
            if pynag_object.name is None:
                new_generic_name = "%s-copy" % smart_str(pynag_object.get_description())
            else:
                new_generic_name = '%s-copy' % smart_str(pynag_object.name)
            self.fields['name'] = forms.CharField(
                initial=new_generic_name,
                help_text=_("Select a new generic name for this %(object_type)s") % {'object_type': object_type}
            )
        elif object_type == 'host':
            new_host_name = "%s-copy" % smart_str(pynag_object.get_description())
            self.fields['host_name'] = forms.CharField(
                help_text=_("Select a new host name for this host"), initial=new_host_name)
            self.fields['address'] = forms.CharField(
                help_text=_("Select a new ip address for this host"))
            self.fields['recursive'] = forms.BooleanField(
                required=False, label="Copy Services", help_text=_("Check this box if you also want to copy all services of this host."))
        elif object_type == 'service':
            service_description = "%s-copy" % smart_str(pynag_object.service_description)
            self.fields['host_name'] = forms.CharField(
                help_text=_("Select a new host name for this service"), initial=pynag_object.host_name)
            self.fields['service_description'] = forms.CharField(
                help_text=_("Select new service description for this service"), initial=service_description)
        else:
            field_name = "%s_name" % object_type
            initial = "%s-copy" % smart_str(pynag_object[field_name])
            help_text = object_definitions[
                object_type][field_name].get('help_text')
            if help_text == '':
                help_text = _("Please specify a new %(field_name)s") % {'field_name': smart_str(field_name)}
            self.fields[field_name] = forms.CharField(
                initial=initial, help_text=help_text)

    def save(self):
        # If copy() returns a single object, lets transform it into a list
        tmp = self.pynag_object.copy(**self.cleaned_data)
        if not isinstance(tmp, list):
            tmp = [tmp]
        self.copied_objects = tmp

    def _clean_shortname(self):
        """ Make sure shortname of a particular object does not exist.

        Raise validation error if shortname is found
        """
        object_type = self.pynag_object.object_type
        field_name = "%s_name" % object_type
        value = smart_str(self.cleaned_data[field_name])
        try:
            self.pynag_object.objects.get_by_shortname(value)
            values = {'object_type': object_type, 'field_name': field_name, 'value': value}
            raise forms.ValidationError(_("A %(object_type)s with %(field_name)s='%(value)s' already exists.") % values)
        except KeyError:
            return value

    def clean_host_name(self):
        if self.pynag_object.object_type == 'service':
            return smart_str(self.cleaned_data['host_name'])
        return self._clean_shortname()

    def clean_timeperiod_name(self):
        return self._clean_shortname()

    def clean_command_name(self):
        return self._clean_shortname()

    def clean_contactgroup_name(self):
        return self._clean_shortname()

    def clean_hostgroup_name(self):
        return self._clean_shortname()

    def clean_servicegroup_name(self):
        return self._clean_shortname()

    def clean_contact_name(self):
        return self._clean_shortname()


class BaseBulkForm(AdagiosForm):

    """ To make changes to multiple objects at once

    * any POST data that has the name change_<OBJECTID> will be fetched
    and the ObjectDefinition saved in self.changed_objects
    * any POST data that has the name hidden_<OBJECTID> will be fetched
    and the ObjectDefinition saved in self.all_objects
    """

    def __init__(self, objects=None, *args, **kwargs):
        self.objects = []
        self.all_objects = []
        self.changed_objects = []
        if not objects:
            objects = []
        forms.Form.__init__(self, *args, **kwargs)
        for k, v in self.data.items():
            if k.startswith('hidden_'):
                obj = Model.ObjectDefinition.objects.get_by_id(v)
                if obj not in self.all_objects:
                    self.all_objects.append(obj)
            if k.startswith('change_'):
                object_id = k[len("change_"):]
                obj = Model.ObjectDefinition.objects.get_by_id(object_id)
                if obj not in self.changed_objects:
                    self.changed_objects.append(obj)
                if obj not in self.all_objects:
                    self.all_objects.append(obj)

    def clean(self):
        #self.cleaned_data = {}
        for k, v in self.data.items():
            if k.startswith('hidden_'):
                self.cleaned_data[k] = v
                obj = Model.ObjectDefinition.objects.get_by_id(v)
                if obj not in self.all_objects:
                    self.all_objects.append(obj)
            if k.startswith('change_'):
                self.cleaned_data[k] = v
                object_id = k[len("change_"):]
                obj = Model.ObjectDefinition.objects.get_by_id(object_id)
                if obj not in self.changed_objects:
                    self.changed_objects.append(obj)
        for k, v in self.cleaned_data.items():
            self.cleaned_data[k] = smart_str(self.cleaned_data[k])
        return self.cleaned_data


class BulkEditForm(BaseBulkForm):
    attribute_name = forms.CharField()
    new_value = forms.CharField()

    def save(self):
        for i in self.changed_objects:
            key = self.cleaned_data['attribute_name']
            value = self.cleaned_data['new_value']
            i[key] = value
            i.save()


class BulkCopyForm(BaseBulkForm):
    attribute_name = forms.CharField()
    new_value = forms.CharField()

    def __init__(self, *args, **kwargs):
        BaseBulkForm.__init__(self, *args, **kwargs)
        self.fields['attribute_name'].value = "test 2"
        # Lets take a look at the first item to be copied and suggest a field
        # name to change

    def save(self):
        for i in self.changed_objects:
            key = self.cleaned_data['attribute_name']
            value = self.cleaned_data['new_value']
            kwargs = {key: value}
            i.copy(**kwargs)


class BulkDeleteForm(BaseBulkForm):

    """ Form used to delete multiple objects at once """
    yes_i_am_sure = forms.BooleanField(label=_("Yes, I am sure"))

    def delete(self):
        """ Deletes every object in the form """
        for i in self.changed_objects:
            if i.object_type == 'host':
                recursive = True
            else:
                recursive = False
            i.delete(recursive=recursive)


class CheckCommandForm(PynagForm):

    def __init__(self, *args, **kwargs):
        super(AdagiosForm, self).__init__(*args, **kwargs)
        self.pynag_object = Model.Service()
        self.fields['host_name'] = self.get_pynagField('host_name')
        self.fields['service_description'] = self.get_pynagField(
            'service_description')
        self.fields['check_command'] = self.get_pynagField('check_command')


class AddTemplateForm(PynagForm):

    """ Use this form to add one template """
    object_type = forms.ChoiceField(choices=ALL_OBJECT_TYPES)
    name = forms.CharField(max_length=100)

    def __init__(self, *args, **kwargs):
        super(PynagForm, self).__init__(*args, **kwargs)

    def clean(self):
        cleaned_data = super(AddTemplateForm, self).clean()
        if "object_type" not in cleaned_data:
            raise forms.ValidationError(_('Object type is required'))
        object_type = cleaned_data['object_type']
        name = cleaned_data['name']
        if object_type not in Model.string_to_class:
            raise forms.ValidationError(
                _("We dont know nothing about how to add a '%(object_type)s'") % {'object_type': object_type})
        objectdefinition = Model.string_to_class.get(object_type)
        # Check if name already exists
        try:
            objectdefinition.objects.get_by_name(name)
            raise forms.ValidationError(
                _("A %(object_type)s with name='%(name)s' already exists.") % {'object_type': object_type,
                                                                               'name': smart_str(name),
                                                                               })
        except KeyError:
            pass
        self.pynag_object = objectdefinition()
        self.pynag_object['register'] = "0"

        return cleaned_data


class AddObjectForm(PynagForm):

    def __init__(self, object_type, initial=None, *args, **kwargs):
        self.pynag_object = Model.string_to_class.get(object_type)()
        super(AdagiosForm, self).__init__(*args, **kwargs)
        # Some object types we will suggest a template:
        if object_type in ('host', 'contact', 'service'):
            self.fields['use'] = self.get_pynagField('use')
            self.fields['use'].initial = self.get_template_if_it_exists()
            self.fields['use'].help_text = _("Inherit attributes from this template")
        if object_type == 'host':
            self.fields['host_name'] = self.get_pynagField('host_name', required=True)
            self.fields['address'] = self.get_pynagField('address', required=True)
            self.fields['alias'] = self.get_pynagField('alias', required=False)
        elif object_type == 'service':
            self.fields['service_description'] = self.get_pynagField('service_description', required=True)
            self.fields['host_name'] = self.get_pynagField('host_name', required=False)
            self.fields['host_name'].help_text = _('Tell us which host this service check will be applied to')
            self.fields['hostgroup_name'] = self.get_pynagField('hostgroup_name', required=False)
            self.fields['hostgroup_name'].help_text = _("If you specify any hostgroups, this service will be applied to all hosts in that hostgroup")
        else:
            field_name = "%s_name" % object_type
            self.fields[field_name] = self.get_pynagField(
                field_name, required=True)
        # For some reason calling super()__init__() with initial as a parameter
        # will not work on PynagChoiceFields. This forces initial value to be set:
        initial = initial or {}
        for field_name, field in self.fields.items():
            initial_value = initial.get(field_name, None)
            if initial_value:
                field.initial = str(initial_value)

    def get_template_if_it_exists(self):
        """Get name of a default template when adding new host/service/contact

        We will use settings.default_{object_type}_template as a parent by default.

        Returns:
            String. Name of default template to use. Empty string if
            settings.default_{object_type}_template does not exist in current config.
        """
        if self.pynag_object.object_type == 'service':
            template_name = adagios.settings.default_service_template
        elif self.pynag_object.object_type == 'host':
            template_name = adagios.settings.default_host_template
        elif self.pynag_object.object_type == 'contact':
            template_name = adagios.settings.default_contact_template
        else:
            template_name = 'generic-%s' % self.pynag_object.object_type

        if self.pynag_object.objects.filter(name=template_name):
            return template_name
        return ''

    @property
    def changed_data(self):
        """Get a list of fields that have changed.

         In the case of new object, we treat every field as a changed field, so that default
         values will be included in the newly created object.

        See https://github.com/opinkerfi/adagios/issues/527
        """
        return self.fields.keys()

    def clean(self):
        cleaned_data = super(AddObjectForm, self).clean()
        if self.pynag_object.object_type == 'service':
            host_name = cleaned_data.get('host_name')
            hostgroup_name = cleaned_data.get('hostgroup_name')
            if host_name in (None, 'None', '') and hostgroup_name in (None, 'None', ''):
                raise forms.ValidationError(_("Please specify either hostgroup_name or host_name"))
        return cleaned_data

    def clean_timeperiod_name(self):
        return self._clean_shortname()

    def clean_command_name(self):
        return self._clean_shortname()

    def clean_contactgroup_name(self):
        return self._clean_shortname()

    def clean_servicegroup_name(self):
        return self._clean_shortname()

    def clean_contact_name(self):
        return self._clean_shortname()

    def clean_host_name(self):
        if self.pynag_object.object_type == 'service':
            value = self.cleaned_data['host_name']
            if not value or value == 'null':
                return None
            hosts = value.split(',')
            for i in hosts:
                existing_hosts = Model.Host.objects.filter(host_name=i)
                if not existing_hosts:
                    raise forms.ValidationError(
                        _("Could not find host called '%(i)s'") % {'i': smart_str(i)})
                return smart_str(self.cleaned_data['host_name'])
        return self._clean_shortname()

    def clean_hostgroup_name(self):
        if self.pynag_object.object_type == 'service':
            value = self.cleaned_data['hostgroup_name']
            if value in (None, '', 'null'):
                return None
            groups = value.split(',')
            for i in groups:
                existing_hostgroups = Model.Hostgroup.objects.filter(hostgroup_name=i)
                if not existing_hostgroups:
                    raise forms.ValidationError(
                        _("Could not find hostgroup called '%(i)s'") % {'i': smart_str(i)})
                return smart_str(self.cleaned_data['hostgroup_name'])
        return self._clean_shortname()

    def _clean_shortname(self):
        """ Make sure shortname of a particular object does not exist.

        Raise validation error if shortname is found
        """
        object_type = self.pynag_object.object_type
        field_name = "%s_name" % object_type
        value = smart_str(self.cleaned_data[field_name])
        try:
            self.pynag_object.objects.get_by_shortname(value)
            raise forms.ValidationError(
                _("A %(object_type)s with %(field_name)s='%(value)s' already exists.") % {'object_type': object_type,
                                                                                          'field_name': field_name,
                                                                                          'value': smart_str(value),
                                                                                          })
        except KeyError:
            return value


class ImportObjectsForm(AdagiosForm):
    object_type = forms.ChoiceField(
        choices=ALL_OBJECT_TYPES, help_text=_('Imported objects are all of this type'), initial='host')
    seperator = forms.CharField(
        help_text=_('Columns are seperated by this letter'), initial=',')
    destination_filename = forms.CharField(
        help_text=_('Save objects in this file. If empty pynag will find the best place to store each object'),
        initial='', required=False)
    objects = forms.CharField(
        widget=forms.Textarea(attrs={'placeholder':_INITIAL_OBJECTS_IMPORT}),
        help_text=_("First line should be column headers, e.g. host_name,address"),
        initial=_INITIAL_OBJECTS_IMPORT,
        )

    def parse_objects_from_form(self):
        cleaned_data = self.clean()
        objects = cleaned_data['objects']
        object_type = cleaned_data['object_type']
        destination_filename = self.cleaned_data['destination_filename']
        seperator = cleaned_data['seperator']
        parsed_objects = importer.parse_csv_string(objects, seperator=seperator)
        pynag_objects = importer.dict_to_pynag_objects(parsed_objects, object_type=object_type)

        for pynag_object in pynag_objects:
            if destination_filename:
                pynag_object.set_filename(destination_filename)
            else:
                pynag_object.set_filename(pynag_object.get_suggested_filename())
        return pynag_objects

    def get_duplicate_pynag_objects(self):
        pynag_objects = self.parse_objects_from_form()
        duplicates = []
        short_names = []
        for pynag_object in pynag_objects:
            short_name = pynag_object.get_description()
            if not short_name:
                continue
            if pynag_object.objects.filter(shortname=short_name) or short_name in short_names:
                duplicates.append(pynag_object)
            short_names.append(short_name)
        return duplicates

    def get_unique_pynag_objects(self):
        objects = self.parse_objects_from_form()
        duplicates = self.get_duplicate_pynag_objects()
        unique_objects = [i for i in objects if i not in duplicates]
        return unique_objects

    def clean_destination_filename(self):
        destination_filename = self.cleaned_data['destination_filename']
        if not destination_filename:
            return None
        if destination_filename not in Model.config.get_cfg_files():
            raise forms.ValidationError(_('Sorry, you can only write files inside current config directories.'))

    def save(self):
        pynag_objects = self.get_unique_pynag_objects()
        for pynag_object in pynag_objects:
            pynag_object.save()
        return pynag_objects



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2010, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" objectbrowser/all_attributes.py

This is an extends of pynag's all_attributes with friendly help message for all attributes.
"""

from pynag.Model.all_attributes import object_definitions
from django.utils.translation import ugettext as _


object_definitions["any"]["use"][
    "help_text"] = _("Specifies which object to inherit settings from")
object_definitions["any"]["register"][
    "help_text"] = _("Specifies if object is active (registered) or not")
object_definitions["any"]["name"][
    "help_text"] = _("Generic name of this objects. Only used for templates.")
object_definitions["host"]["host_name"]["help_text"] = _("e.g. web01.example.com")
object_definitions["host"]["alias"]["help_text"] = _("e.g. My Monitored Host")
object_definitions["host"]["display_name"]["help_text"] = _(" ")
object_definitions["host"]["address"]["help_text"] = _("e.g. 127.0.0.1")
object_definitions["host"]["parents"][
    "help_text"] = _("Network parents of this host. No notification will be sent if parent is down.")
object_definitions["host"]["hostgroups"][
    "help_text"] = _("Which hostgroups this host belongs to")
object_definitions["host"]["check_command"][
    "help_text"] = _("Command to execute when this object is checked")
object_definitions["host"]["initial_state"][
    "help_text"] = _('By default Nagios will assume that all hosts are in UP states when it starts. You can override the initial state for a host by using this directive. Valid options are: o = UP, d = DOWN, and u = UNREACHABLE.')
object_definitions["host"]["max_check_attempts"][
    "help_text"] = _("How many failures do occur before notifications will be sent")
object_definitions["host"]["check_interval"][
    "help_text"] = _("How many minutes to wait between checks")
object_definitions["host"]["retry_interval"][
    "help_text"] = _("How many minutes to wait between checks when object goes to warning or critical state")
object_definitions["host"]["active_checks_enabled"][
    "help_text"] = _("Whether Nagios actively checks this host")
object_definitions["host"]["passive_checks_enabled"][
    "help_text"] = _("Whether Nagios passively accepts check results from an external source")
object_definitions["host"]["check_period"][
    "help_text"] = _("When nagios checks for this host")
object_definitions["host"]["obsess_over_host"][
    "help_text"] = _('This directive determines whether or not checks for the host will be "obsessed" over using the ochp_command.')
object_definitions["host"]["check_freshness"]["help_text"] = _(" ")
object_definitions["host"]["freshness_threshold"]["help_text"] = _(" ")
object_definitions["host"]["event_handler"]["help_text"] = _(" ")
object_definitions["host"]["event_handler_enabled"]["help_text"] = _(" ")
object_definitions["host"]["low_flap_threshold"]["help_text"] = _(" ")
object_definitions["host"]["high_flap_threshold"]["help_text"] = _(" ")
object_definitions["host"]["flap_detection_enabled"]["help_text"] = _(" ")
object_definitions["host"]["flap_detection_options"]["help_text"] = _(" ")
object_definitions["host"]["process_perf_data"]["help_text"] = _(" ")
object_definitions["host"]["retain_status_information"]["help_text"] = _(" ")
object_definitions["host"]["retain_nonstatus_information"]["help_text"] = _(" ")
object_definitions["host"]["contacts"]["help_text"] = _(" ")
object_definitions["host"]["contact_groups"]["help_text"] = _(" ")
object_definitions["host"]["notification_interval"]["help_text"] = _(" ")
object_definitions["host"]["first_notification_delay"]["help_text"] = _(" ")
object_definitions["host"]["notification_period"]["help_text"] = _(" ")
object_definitions["host"]["notification_options"]["help_text"] = _(" ")
object_definitions["host"]["notifications_enabled"]["help_text"] = _(" ")
object_definitions["host"]["stalking_options"]["help_text"] = _(" ")
object_definitions["host"]["notes"]["help_text"] = _(" ")
object_definitions["host"]["notes_url"]["help_text"] = _(" ")
object_definitions["host"]["action_url"]["help_text"] = _(" ")
object_definitions["host"]["icon_image"]["help_text"] = _(" ")
object_definitions["host"]["icon_image_alt"]["help_text"] = _(" ")
object_definitions["host"]["vrml_image"]["help_text"] = _(" ")
object_definitions["host"]["statusmap_image"]["help_text"] = _(" ")
object_definitions["host"]["2d_coords"]["help_text"] = _(" ")
object_definitions["host"]["3d_coords"]["help_text"] = _(" ")
object_definitions["hostgroup"]["hostgroup_name"][
    "help_text"] = _("Unique name for this hostgroup (e.g. webservers)")
object_definitions["hostgroup"]["alias"][
    "help_text"] = _("Human friendly name (e.g. My Web Servers)")
object_definitions["hostgroup"]["members"][
    "help_text"] = _("List of hosts that belong to this group")
object_definitions["hostgroup"]["hostgroup_members"][
    "help_text"] = _("List of hostgroups that belong to this group")
object_definitions["hostgroup"]["notes"][
    "help_text"] = _("You can put your custom notes here for your hostgroup")
object_definitions["hostgroup"]["notes_url"][
    "help_text"] = _("Type in an url for example to a documentation site for this hostgroup")
object_definitions["hostgroup"]["action_url"]["help_text"] = _(" ")
object_definitions["service"]["host_name"][
    "help_text"] = _("e.g. web01.example.com")
object_definitions["service"]["hostgroup_name"][
    "help_text"] = _("Hostgroup this service belongs to")
object_definitions["service"]["service_description"][
    "help_text"] = _("e.g. 'Disk Status'")
object_definitions["service"]["display_name"]["help_text"] = _(" ")
object_definitions["service"]["servicegroups"][
    "help_text"] = _("Servicegroups that this service belongs to")
object_definitions["service"]["is_volatile"]["help_text"] = _(" ")
object_definitions["service"]["check_command"][
    "help_text"] = _("Command that is executed when this service is checked")
object_definitions["service"]["initial_state"]["help_text"] = _(" ")
object_definitions["service"]["max_check_attempts"][
    "help_text"] = _("How many times to try before failure notifications are sent out")
object_definitions["service"]["check_interval"][
    "help_text"] = _("How many minutes to wait between checks")
object_definitions["service"]["retry_interval"][
    "help_text"] = _("How many minutes to wait between checks when failure occurs")
object_definitions["service"]["active_checks_enabled"][
    "help_text"] = _("Enable if you want nagios to actively check this service")
object_definitions["service"]["passive_checks_enabled"][
    "help_text"] = _("Enable if you want nagios to passively accept check results from an external source")
object_definitions["service"]["check_period"][
    "help_text"] = _("Period which this service is checked.")
object_definitions["service"]["obsess_over_service"]["help_text"] = _(" ")
object_definitions["service"]["check_freshness"]["help_text"] = _(" ")
object_definitions["service"]["freshness_threshold"]["help_text"] = _(" ")
object_definitions["service"]["event_handler"]["help_text"] = _(" ")
object_definitions["service"]["event_handler_enabled"]["help_text"] = _(" ")
object_definitions["service"]["low_flap_threshold"]["help_text"] = _(" ")
object_definitions["service"]["high_flap_threshold"]["help_text"] = _(" ")
object_definitions["service"]["flap_detection_enabled"]["help_text"] = _(" ")
object_definitions["service"]["flap_detection_options"]["help_text"] = _(" ")
object_definitions["service"]["process_perf_data"]["help_text"] = _(" ")
object_definitions["service"]["retain_status_information"]["help_text"] = _(" ")
object_definitions["service"]["retain_nonstatus_information"]["help_text"] = _(" ")
object_definitions["service"]["notification_interval"]["help_text"] = _(" ")
object_definitions["service"]["first_notification_delay"]["help_text"] = _(" ")
object_definitions["service"]["notification_period"][
    "help_text"] = _("Period which notifications are sent out for this service")
object_definitions["service"]["notification_options"]["help_text"] = _(" ")
object_definitions["service"]["notifications_enabled"]["help_text"] = _(" ")
object_definitions["service"]["contacts"][
    "help_text"] = _("Which contacts to notify if service fails")
object_definitions["service"]["contact_groups"][
    "help_text"] = _("Which contactgroups to send notifications to if service fails")
object_definitions["service"]["stalking_options"]["help_text"] = _(" ")
object_definitions["service"]["notes"]["help_text"] = _(" ")
object_definitions["service"]["notes_url"]["help_text"] = _(" ")
object_definitions["service"]["action_url"]["help_text"] = _(" ")
object_definitions["service"]["icon_image"]["help_text"] = _(" ")
object_definitions["service"]["icon_image_alt"]["help_text"] = _(" ")
object_definitions["servicegroup"]["servicegroup_name"][
    "help_text"] = _("Unique name for this service group")
object_definitions["servicegroup"]["alias"][
    "help_text"] = _("Human friendly name for this servicegroup")
object_definitions["servicegroup"]["members"][
    "help_text"] = _("List of services that belong to this group (Example: localhost,CPU Utilization,localhost,Disk Usage)")
object_definitions["servicegroup"]["servicegroup_members"][
    "help_text"] = _("Servicegroups that are members of this servicegroup")
object_definitions["servicegroup"]["notes"][
    "help_text"] = _("Arbitrary notes or description of this servicegroup")
object_definitions["servicegroup"]["notes_url"][
    "help_text"] = _("Arbitrary url to a site of your choice")
object_definitions["servicegroup"]["action_url"][
    "help_text"] = _("Arbitrary url to a site of your choice")
object_definitions["contact"]["contact_name"][
    "help_text"] = _("Unique name for this contact (e.g. username@domain.com)")
object_definitions["contact"]["alias"][
    "help_text"] = _("Human Friendly Name for this contact (e.g. Full Name)")
object_definitions["contact"]["contactgroups"][
    "help_text"] = _("List of groups that this contact is a member of.")
object_definitions["contact"]["host_notifications_enabled"][
    "help_text"] = _("If this contact will receive host notifications.")
object_definitions["contact"]["service_notifications_enabled"][
    "help_text"] = _("If this contact will receive service notifications.")
object_definitions["contact"]["host_notification_period"][
    "help_text"] = _("When will this contact receive host notifications")
object_definitions["contact"]["service_notification_period"][
    "help_text"] = _("When will this contact receive service notifications")
object_definitions["contact"]["host_notification_options"][
    "help_text"] = _("Which host notifications this contact will receive")
object_definitions["contact"]["service_notification_options"][
    "help_text"] = _("Which service notifications this contact will receive")
object_definitions["contact"]["host_notification_commands"][
    "help_text"] = _("What command will be used to send host notifications to this contact")
object_definitions["contact"]["service_notification_commands"][
    "help_text"] = _("What command will be used to send service notifications to this contact")
object_definitions["contact"]["email"][
    "help_text"] = _("E-mail address of this contact")
object_definitions["contact"]["pager"][
    "help_text"] = _("Pager number of this contact")
object_definitions["contact"]["address"][
    "help_text"] = _("Address of this contact")
object_definitions["contact"]["can_submit_commands"][
    "help_text"] = _("If this contact is able to submit commands to nagios command pipe")
object_definitions["contact"]["retain_status_information"]["help_text"] = _(" ")
object_definitions["contact"]["retain_nonstatus_information"]["help_text"] = _(" ")
object_definitions["contactgroup"]["contactgroup_name"][
    "help_text"] = _("Unique name for this contact group (e.g. 'webservers')")
object_definitions["contactgroup"]["alias"][
    "help_text"] = _("Human Friendly Name (e.g. 'My Web Servers')")
object_definitions["contactgroup"]["members"][
    "help_text"] = _("Every Contact listed here will be a member of this contactgroup")
object_definitions["contactgroup"]["contactgroup_members"][
    "help_text"] = _("Every Contactgroup listed here will be a member of this contactgroup")
object_definitions["timeperiod"]["timeperiod_name"][
    "help_text"] = _("Unique name for this timeperiod (.e.g. 'workhours')")
object_definitions["timeperiod"]["alias"][
    "help_text"] = _("Human Friendly name for this timeperiod")
object_definitions["timeperiod"]["[weekday]"]["help_text"] = _(" ")
object_definitions["timeperiod"]["[exception]"]["help_text"] = _(" ")
object_definitions["timeperiod"]["exclude"]["help_text"] = _(" ")
object_definitions["command"]["command_name"][
    "help_text"] = _("Unique name for this command")
object_definitions["command"]["command_line"][
    "help_text"] = _("Command line of the command that will be executed")
object_definitions["servicedependency"][
    "dependent_host_name"]["help_text"] = _(" ")
object_definitions["servicedependency"][
    "dependent_hostgroup_name"]["help_text"] = _(" ")
object_definitions["servicedependency"][
    "dependent_service_description"]["help_text"] = _(" ")
object_definitions["servicedependency"]["host_name"]["help_text"] = _(" ")
object_definitions["servicedependency"]["hostgroup_name"]["help_text"] = _(" ")
object_definitions["servicedependency"][
    "service_description"]["help_text"] = _(" ")
object_definitions["servicedependency"]["inherits_parent"]["help_text"] = _(" ")
object_definitions["servicedependency"][
    "execution_failure_criteria"]["help_text"] = _(" ")
object_definitions["servicedependency"][
    "notification_failure_criteria"]["help_text"] = _(" ")
object_definitions["servicedependency"]["dependency_period"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["host_name"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["hostgroup_name"]["help_text"] = _(" ")
object_definitions["serviceescalation"][
    "service_description"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["contacts"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["contact_groups"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["first_notification"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["last_notification"]["help_text"] = _(" ")
object_definitions["serviceescalation"][
    "notification_interval"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["escalation_period"]["help_text"] = _(" ")
object_definitions["serviceescalation"]["escalation_options"]["help_text"] = _(" ")
object_definitions["hostdependency"]["dependent_host_name"]["help_text"] = _(" ")
object_definitions["hostdependency"][
    "dependent_hostgroup_name"]["help_text"] = _(" ")
object_definitions["hostdependency"]["host_name"]["help_text"] = _(" ")
object_definitions["hostdependency"]["hostgroup_name"]["help_text"] = _(" ")
object_definitions["hostdependency"]["inherits_parent"]["help_text"] = _(" ")
object_definitions["hostdependency"][
    "execution_failure_criteria"]["help_text"] = _(" ")
object_definitions["hostdependency"][
    "notification_failure_criteria"]["help_text"] = _(" ")
object_definitions["hostdependency"]["dependency_period"]["help_text"] = _(" ")
object_definitions["hostescalation"]["host_name"]["help_text"] = _(" ")
object_definitions["hostescalation"]["hostgroup_name"]["help_text"] = _(" ")
object_definitions["hostescalation"]["contacts"]["help_text"] = _(" ")
object_definitions["hostescalation"]["contact_groups"]["help_text"] = _(" ")
object_definitions["hostescalation"]["first_notification"]["help_text"] = _(" ")
object_definitions["hostescalation"]["last_notification"]["help_text"] = _(" ")
object_definitions["hostescalation"]["notification_interval"]["help_text"] = _(" ")
object_definitions["hostescalation"]["escalation_period"]["help_text"] = _(" ")
object_definitions["hostescalation"]["escalation_options"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["host_name"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["notes"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["notes_url"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["action_url"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["icon_image"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["icon_image_alt"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["vrml_image"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["statusmap_image"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["2d_coords"]["help_text"] = _(" ")
object_definitions["hostextinfo"]["3d_coords"]["help_text"] = _(" ")
object_definitions["serviceextinfo"]["host_name"]["help_text"] = _(" ")
object_definitions["serviceextinfo"]["service_description"]["help_text"] = _(" ")
object_definitions["serviceextinfo"]["notes"]["help_text"] = _(" ")
object_definitions["serviceextinfo"]["notes_url"]["help_text"] = _(" ")
object_definitions["serviceextinfo"]["action_url"]["help_text"] = _(" ")
object_definitions["serviceextinfo"]["icon_image"]["help_text"] = _(" ")
object_definitions["serviceextinfo"]["icon_image_alt"]["help_text"] = _(" ")
